@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/calma/CalmaWrite.c,v 1.7 2010/06/24 12:37:15 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/calma/CalmaWrite.c,v 1.8 2010/12/22 16:29:06 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -453,9 +453,15 @@ calmaProcessDef(def, outf)
 	fi = PaOpen(filename, "r", "", Path, CellLibPath, (char **)NULL);
 	if (fi == NULL)
 	{
-	    TxError("Calma output error:  Can't find GDS file for vendor cell."
-	    		"  Using magic's internal definition\n");
-	    isReadOnly = FALSE;
+	    /* This is a rare error, but if the subcell is inside	*/
+	    /* another vendor GDS, it would not normally be output.	*/
+
+	    DBPropGet(def->cd_parents->cu_parent, "GDS_FILE", &isReadOnly);
+	    if (!isReadOnly)
+		TxError("Calma output error:  Can't find GDS file for vendor cell."
+	    			"  Using magic's internal definition\n");
+	    else 
+		def->cd_flags |= CDVENDORGDS;
 	}
 	else
 	{
