@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdcl.c,v 1.2 2009/05/03 15:01:04 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdcl.c,v 1.5 2010/08/25 17:33:55 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -600,13 +600,8 @@ cifPaintCurrentFunc(tile, type)
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (DBPaintOnPlane(type, pNum))
 	{
-#ifdef NONMANHATTAN
 	    DBNMPaintPlane(cifReadCellDef->cd_planes[pNum], TiGetTypeExact(tile),
 		    &area, DBStdPaintTbl(type, pNum), (PaintUndoInfo *) NULL);
-#else
-	    DBPaintPlane(cifReadCellDef->cd_planes[pNum], &area,
-		    DBStdPaintTbl(type, pNum), (PaintUndoInfo *) NULL);
-#endif
 	}
 
     return  0;		/* To keep the search alive. */
@@ -950,7 +945,13 @@ cifParseUser94()
     }
     if (type >=0 )
     {
-    	 (void) DBPutLabel(cifReadCellDef, &rectangle, -1, name, type, 0);
+	int flags;
+	if (layer >= 0 && (cifCurReadStyle->crs_layers[layer]->crl_flags
+		& CIFR_TEXTLABELS))
+	    flags = LABEL_STICKY;
+	else
+	    flags = 0;
+    	(void) DBPutLabel(cifReadCellDef, &rectangle, -1, name, type, flags);
     }
     freeMagic(name);
     return TRUE;
@@ -1051,7 +1052,13 @@ cifParseUser95()
     else type = TT_SPACE;
     if (type >=0 )
     {
-    	 (void) DBPutLabel(cifReadCellDef, &rectangle, -1, name, type, 0);
+	int flags;
+	if (layer >= 0 && (cifCurReadStyle->crs_layers[layer]->crl_flags
+		& CIFR_TEXTLABELS))
+	    flags = LABEL_STICKY;
+	else
+	    flags = 0;
+    	(void) DBPutLabel(cifReadCellDef, &rectangle, -1, name, type, flags);
     }
 
     freeMagic(name);
@@ -1178,7 +1185,7 @@ CIFReadCellCleanup()
 		    TiFreePlane(gdsplanes[pNum]);
 		}
 	    }
-	    freeMagic(def->cd_client);
+	    freeMagic((char *)def->cd_client);
 
 	    if (def->cd_parents != (CellUse *)NULL)
 	    {
