@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdtech.c,v 1.3 2010/08/25 17:33:55 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdtech.c,v 1.4 2010/09/15 15:45:30 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -572,7 +572,7 @@ CIFReadTechLine(sectionName, argc, argv)
 
 	if (argc >= 3)
 	{
-	    if(!strcmp(argv[argc - 1], "nanometers"))
+	    if(!strncmp(argv[argc - 1], "nanom", 5))
 		cifCurReadStyle->crs_multiplier = 10;
 	}
 
