@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFtech.c,v 1.5 2008/12/04 17:10:29 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFtech.c,v 1.7 2010/10/20 20:34:19 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -683,7 +683,7 @@ CIFTechLine(sectionName, argc, argv)
 
 	if (argc >= 3)
 	{
-	    if (strcmp(argv[argc - 1], "nanometers") == 0)
+	    if (strncmp(argv[argc - 1], "nanom", 5) == 0)
 		CIFCurStyle->cs_expander *= 10;
 	    else if (strncmp(argv[argc - 1], "angstr", 6) == 0)
 		CIFCurStyle->cs_expander *= 100;
@@ -2193,6 +2193,9 @@ CIFTechOutputScale(n, d)
 	}
     }
 
+    /* Recompute drc-cif rule distances */
+    drcCifScale(d, 1);
+
     /* Reduce the scale and all distances by the greatest common	*/
     /* factor of everything.						*/
 
@@ -2275,6 +2278,9 @@ CIFTechOutputScale(n, d)
 	}
     }
 
+    /* Recompute drc-cif rule distances */
+    drcCifScale(1, lexpand);
+
     /* Recompute value of cs_radius */
     cifComputeHalo(ostyle);
 }
