@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdLQ.c,v 1.8 2008/12/11 18:36:43 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdLQ.c,v 1.9 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -221,6 +221,8 @@ CmdLabel(w, cmd)
 
     if (font >= 0)
     {
+	char *yp = NULL;
+
 	size = DBLambda[1];
 	if (cmd->tx_argc > 3)
 	    if (StrIsNumeric(cmd->tx_argv[3]))
@@ -231,18 +233,45 @@ CmdLabel(w, cmd)
 		rotate = atoi(cmd->tx_argv[4]);
 
 	if (cmd->tx_argc > 6)
-	    if (StrIsNumeric(cmd->tx_argv[5]) && StrIsNumeric(cmd->tx_argv[6]))
+	{
+	    if ((yp = strchr(cmd->tx_argv[5], ' ')) != NULL)
+	    {
+		*yp = '\0';
+		yp++;
+		if (StrIsNumeric(cmd->tx_argv[5]) && StrIsNumeric(yp))
+		{
+		    offx = cmdScaleCoord(w, cmd->tx_argv[5], TRUE, TRUE, 8);
+		    offy = cmdScaleCoord(w, yp, TRUE, FALSE, 8);
+		    *yp = ' ';
+		}
+		else
+		{
+		    TxError("Uninterpretable offset value \"%s %s\"\n",
+				cmd->tx_argv[5], yp);
+		    *yp = ' ';
+		    return;
+		}
+	    }
+	    else if (StrIsNumeric(cmd->tx_argv[5]) && StrIsNumeric(cmd->tx_argv[6]))
 	    {
 		/* offx and offy are entered in lambda units and	*/
-		/* re-interpreted as quarter-internal units.		*/
+		/* re-interpreted as eighth-internal units.		*/
 
-		offx = cmdScaleCoord(w, cmd->tx_argv[5], TRUE, TRUE, 4);
-		offy = cmdScaleCoord(w, cmd->tx_argv[6], TRUE, FALSE, 4);
+		offx = cmdScaleCoord(w, cmd->tx_argv[5], TRUE, TRUE, 8);
+		offy = cmdScaleCoord(w, cmd->tx_argv[6], TRUE, FALSE, 8);
 	    }
+	    else
+	    {
+		TxError("Uninterpretable offset value \"%s %s\"\n", cmd->tx_argv[5],
+			cmd->tx_argv[6]);
+		return;
+	    }
+	}
 
-	if (cmd->tx_argc > 7)
+	if (((yp != NULL) && (cmd->tx_argc > 6))  || (cmd->tx_argc > 7))
 	{
-	    pos = GeoNameToPos(cmd->tx_argv[7], FALSE, TRUE);
+	    int pidx = (yp != NULL) ? 6 : 7;
+	    pos = GeoNameToPos(cmd->tx_argv[pidx], FALSE, TRUE);
 	    if (pos < 0)
 		return;
 	    else
