@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBio.c,v 1.4 2008/12/17 03:37:37 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBio.c,v 1.5 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -492,41 +492,25 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
 	 * in the file beginning with 'r'.
 	 */
 nextrect:
-#ifdef NONMANHATTAN 
 	while (((c = getc(f)) == 'r') || (c == 't'))
 	{
 	    TileType dinfo;
 	    int dir;
-#else
-	while (((c = getc(f)) == 'r') || (c == 't'))
-	{
-#endif
 	    /*
 	     * GetRect actually reads the rest of the line up to
 	     * a trailing newline or EOF.
 	     */
 	    if (c == 't')
 	    {
-#ifdef NONMANHATTAN  
 		if ((dir = GetRect(f, 3, rp, n, d)) == 0) goto badfile;
 		dir >>= 1;
 		dinfo = TT_DIAGONAL | ((dir & 0x2) ? TT_SIDE : 0) |
 			((((dir & 0x2) >> 1) ^ (dir & 0x1)) ?
 			TT_DIRECTION : 0);
-#else
-		if (!GetRect(f, 3, rp, n, d)) goto badfile;
-		TxPrintf("Error: cell %s contains non-manhattan geometry\n",
-			cellDef->cd_name);
-		TxPrintf("unsupported by this magic executable.  Skipping.\n");
-		fflush(stdout);
-		continue;
-#endif
 	    }
             else  
 	    {
-#ifdef NONMANHATTAN  
 		dinfo = 0;
-#endif
 		if (!GetRect(f, 4, rp, n, d)) goto badfile;
 	    }
 
@@ -561,10 +545,8 @@ nextrect:
 		    if (TTMaskHasType(rmask, rtype))
 		    {
 			loctype = rtype;
-#ifdef NONMANHATTAN
 			if (dinfo & TT_SIDE) loctype <<= 14;
 			loctype |= dinfo;
-#endif
 			DBPaint(cellDef, rp, loctype);
 		    }
 		}
@@ -2633,7 +2615,6 @@ dbWritePaintFunc(tile, cdarg)
     TileType type = TiGetType(tile);
     TileTypeBitMask *lMask, *rMask;
 
-#ifdef NONMANHATTAN
     int dir;
 
     if (IsSplit(tile))      
@@ -2658,9 +2639,7 @@ dbWritePaintFunc(tile, cdarg)
 	else
 	    return 0;
     }
-    else  
-#endif   
-    if (type != arg->wa_type)
+    else if (type != arg->wa_type)
     {
 	rMask = DBResidueMask(type);
 	if ((type < DBNumUserLayers) ||
@@ -2677,7 +2656,6 @@ dbWritePaintFunc(tile, cdarg)
 	arg->wa_found = TRUE;
     }
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	static char *pos_diag[] = {"nw", "sw", "se", "ne"};
@@ -2688,9 +2666,7 @@ dbWritePaintFunc(tile, cdarg)
 		pos_diag[dir]);
     }
     else
-#endif
-
-    sprintf(pstring, "rect %d %d %d %d\n",
+	sprintf(pstring, "rect %d %d %d %d\n",
 	    LEFT(tile) / arg->wa_reducer, BOTTOM(tile) / arg->wa_reducer,
 	    RIGHT(tile) / arg->wa_reducer, TOP(tile) / arg->wa_reducer);
     FPRINTR(arg->wa_file,pstring);
