@@ -22,7 +22,7 @@
 /* #define	PAINTDEBUG /* For debugging */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBpaint.c,v 1.14 2010/08/25 17:33:55 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBpaint.c,v 1.15 2010/09/24 19:53:19 tim Exp $";
 #endif  /* not lint */
 
 #include <sys/types.h>
@@ -287,13 +287,14 @@ enumerate:
 	if (SigInterruptPending)
 	    break;
 
+	clipTop = TOP(tile);
+	if (clipTop > area->r_ytop) clipTop = area->r_ytop;
+
 	/* Skip processed tiles, if the "mark" option was chosen */
 	if (mark)
 	    if (tile->ti_client != (ClientData) CLIENTDEFAULT)
 		goto paintdone;
 
-	clipTop = TOP(tile);
-	if (clipTop > area->r_ytop) clipTop = area->r_ytop;
 	oldType = TiGetTypeExact(tile);
 
 #ifdef	PAINTDEBUG
@@ -509,7 +510,7 @@ clipdone:
 		if (undo && UndoIsEnabled())
 		    DBPAINTUNDO(tile, newType, undo);
 		TiSetBody(tile, newType);
-	 	if (mark) tile->ti_client = (ClientData)1;
+	 	// if (mark) tile->ti_client = (ClientData)1;
 		/* Reinstate the left and right merge requirements */
 		mergeFlags |= MRG_LEFT;
 		if (RIGHT(tile) >= area->r_xtop) mergeFlags |= MRG_RIGHT;
@@ -530,7 +531,8 @@ clipdone:
 	    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
 		if (TiGetTypeExact(tp) == newType)
 		{
-		    tile = dbPaintMerge(tile, newType, plane, mergeFlags, undo, mark);
+		    tile = dbPaintMerge(tile, newType, area, plane, mergeFlags,
+					undo, mark);
 		    goto paintdone;
 		}
 	    mergeFlags &= ~MRG_LEFT;
@@ -540,7 +542,8 @@ clipdone:
 	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
 		if (TiGetTypeExact(tp) == newType)
 		{
-		    tile = dbPaintMerge(tile, newType, plane, mergeFlags, undo, mark);
+		    tile = dbPaintMerge(tile, newType, area, plane, mergeFlags,
+					undo, mark);
 		    goto paintdone;
 		}
 	    mergeFlags &= ~MRG_RIGHT;
@@ -633,6 +636,9 @@ done:
 enum2:
 	while (TOP(tile) > area->r_ybot)
 	{
+	    clipTop = TOP(tile);
+	    if (clipTop > area->r_ytop) clipTop = area->r_ytop;
+
 	    tile->ti_client = (ClientData)CLIENTDEFAULT;
 
 	    /* Move right if possible */
@@ -662,10 +668,11 @@ enum2:
 		    tile = tpnew;
 		    goto enum2;
 		}
+	        tile->ti_client = (ClientData)CLIENTDEFAULT;
 	    }
 	    /* At left edge -- walk down to next tile along the left edge */
 	    for (tile = LB(tile); RIGHT(tile) <= area->r_xbot; tile = TR(tile))
-		/* Nothing */;
+	        tile->ti_client = (ClientData)CLIENTDEFAULT;
 	}
     }
 
@@ -1758,6 +1765,33 @@ dbNMEnumFunc(tile, arg)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbMarkClient --
+ *
+ *	Mark a tile's client record for use in tracking if this tile has
+ *	been handled in this pass of dbPaintPlane.  If the tile's area
+ *	is outside of "clip", then we don't mark it, or else it will be
+ *	missed during cleanup.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+dbMarkClient(tile, clip)
+    Tile *tile;
+    Rect *clip;
+{
+    if (LEFT(tile) < clip->r_xtop &&
+		RIGHT(tile) > clip->r_xbot &&
+		BOTTOM(tile) < clip->r_ytop &&
+		TOP(tile) > clip->r_ybot)
+	tile->ti_client = (ClientData)1;
+    else
+	tile->ti_client = (ClientData)CLIENTDEFAULT;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1795,9 +1829,10 @@ dbNMEnumFunc(tile, arg)
  */
 
 Tile *
-dbPaintMerge(tile, newType, plane, mergeFlags, undo, mark)
+dbPaintMerge(tile, newType, area, plane, mergeFlags, undo, mark)
     Tile *tile;	/* Tile to be merged with its neighbors */
     TileType newType;	/* Type to which we will change 'tile' */
+    Rect *area;			/* Original area painted, needed for marking */
     Plane *plane;		/* Plane on which this resides */
     int mergeFlags;		/* Specify which directions to merge */
     PaintUndoInfo *undo;	/* See DBPaintPlane() above */
@@ -1886,7 +1921,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo, mark)
 	DBPAINTUNDO(tile, newType, undo);
 
     TiSetBody(tile, newType);
-    if (mark) tile->ti_client = (ClientData)1;
+    if (mark) dbMarkClient(tile, area);
 
 #ifdef	PAINTDEBUG
     if (dbPaintDebug)
@@ -1906,7 +1941,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo, mark)
 	{
 	    tpLast = TiSplitY(tp, TOP(tile));
 	    TiSetBody(tpLast, newType);
-	    if (mark) tile->ti_client = (ClientData)1;
+	    if (mark) dbMarkClient(tile, area);
 	}
 	if (BOTTOM(tp) < BOTTOM(tile)) tp = TiSplitY(tp, BOTTOM(tile));
 	TiJoinX(tile, tp, plane);
@@ -1922,7 +1957,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo, mark)
 	{
 	    tpLast = TiSplitY(tp, TOP(tile));
 	    TiSetBody(tpLast, newType);
-	    if (mark) tile->ti_client = (ClientData)1;
+	    if (mark) dbMarkClient(tile, area);
 	}
 	if (BOTTOM(tp) < BOTTOM(tile)) tp = TiSplitY(tp, BOTTOM(tile));
 	TiJoinX(tile, tp, plane);
