@@ -20,11 +20,12 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCbasic.c,v 1.5 2010/06/24 12:37:16 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCbasic.c,v 1.7 2010/09/20 21:13:22 tim Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <string.h>		// for memcpy()
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -33,6 +34,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "drc/drc.h"
 #include "utils/signals.h"
 #include "utils/maxrect.h"
+#include "utils/malloc.h"
 
 int dbDRCDebug = 0;
 
@@ -128,8 +130,6 @@ point_to_segment(px, py, s1x, s1y, s2x, s2y)
  *
  * Results:
  *	Zero (so that the search will continue).
- *	1 if the rule is a trigger rule and an error rectangle is present
- *	(the trigger rule does not create error paint).
  *
  * Side effects:
  *      Applies the function passed as an argument.
@@ -239,11 +239,32 @@ areaCheck(tile, arg)
     }
 
     if (arg->dCD_cptr->drcc_flags & DRC_TRIGGER)
-	return 1;
+    {
+	Rect *newrlist;
+	int entries = arg->dCD_entries;
 
-    (*(arg->dCD_function))(arg->dCD_celldef, &rect, arg->dCD_cptr,
+	/* The following code allows the rect list to be expanded by	*/
+	/* multiples of 8, when necessary.				*/
+
+	arg->dCD_entries++;
+	if (arg->dCD_rlist == NULL)
+	    arg->dCD_rlist = (Rect *)mallocMagic(8 * sizeof(Rect));
+	else if ((arg->dCD_entries & ~(entries | 7)) == arg->dCD_entries)
+	{
+	    newrlist = (Rect *)mallocMagic((arg->dCD_entries << 1) * sizeof(Rect));
+	    memcpy((void *)newrlist, (void *)arg->dCD_rlist, (size_t)entries *
+			sizeof(Rect));
+	    freeMagic(arg->dCD_rlist);
+	    arg->dCD_rlist = newrlist;
+	}
+	arg->dCD_rlist[arg->dCD_entries - 1] = rect;
+    }
+    else
+    {
+	(*(arg->dCD_function))(arg->dCD_celldef, &rect, arg->dCD_cptr,
 			arg->dCD_clientData);
-    (*(arg->dCD_errors))++;
+	(*(arg->dCD_errors))++;
+    }
     return 0;
 }
 
@@ -315,6 +336,8 @@ DRCBasicCheck (celldef, checkRect, clipRect, function, cdata)
     arg.dCD_function = function;
     arg.dCD_clip = clipRect;
     arg.dCD_clientData = cdata;
+    arg.dCD_rlist = NULL;
+    arg.dCD_entries = 0;
 
     for (planeNum = PL_TECHDEPBASE; planeNum < DBNumPlanes; planeNum++)
     {
@@ -324,6 +347,7 @@ DRCBasicCheck (celldef, checkRect, clipRect, function, cdata)
 		checkRect, &DBAllTypeBits, drcTile, (ClientData) &arg);
     }
     drcCifCheck(&arg);
+    if (arg.dCD_rlist != NULL) freeMagic(arg.dCD_rlist);
     return (errors);
 }
 
@@ -358,7 +382,8 @@ drcTile (tile, arg)
     Rect errRect;		/* Area checked for an individual rule */
     MaxRectsData *mrd;		/* Used by widespacing rule */
     TileTypeBitMask tmpMask, *rMask;
-    bool trigpending;	/* Hack for widespacing rule */
+    bool trigpending;		/* Hack for widespacing rule */
+    bool firsttile;
     int triggered;
     int cdist, dist, ccdist, result;
 
@@ -458,6 +483,7 @@ drcTile (tile, arg)
         int bottom = MAX(BOTTOM(tile), rect->r_ybot);
 	int edgeX = LEFT(tile);
 
+	firsttile = TRUE;
         for (tpleft = BL(tile); BOTTOM(tpleft) < top; tpleft = RT(tpleft))
         {
 	    /* Get the tile types to the left and right of the edge */
@@ -508,7 +534,8 @@ drcTile (tile, arg)
 
 		if (cptr->drcc_flags & DRC_AREA)
 		{
-		    drcCheckArea(tile, arg, cptr);
+		    if (firsttile)
+			drcCheckArea(tile, arg, cptr);
 		    continue;
 		}
 
@@ -523,8 +550,10 @@ drcTile (tile, arg)
 
 		    if (cptr->drcc_flags & DRC_REVERSE)
 			mrd = drcCanonicalMaxwidth(tpleft, GEO_WEST, arg, cptr);
-		    else
+		    else if (firsttile)
 			mrd = drcCanonicalMaxwidth(tile, GEO_EAST, arg, cptr);
+		    else
+			mrd = NULL;
 		    if (!trigpending) cptr->drcc_dist--;
 		    if (trigpending)
 		    {
@@ -552,13 +581,16 @@ drcTile (tile, arg)
 		else if (cptr->drcc_flags & DRC_MAXWIDTH)
 		{
 		    /* bends_illegal option only */
-		    drcCheckMaxwidth(tile, arg, cptr);
+		    if (firsttile)
+			drcCheckMaxwidth(tile, arg, cptr);
 		    continue;
 		}
+		else if (!triggered) mrd = NULL;
+
 		if (cptr->drcc_flags & DRC_RECTSIZE)
 		{
 		    /* only checked for bottom-left tile in a rect area */
-		    if (!TTMaskHasType(&cptr->drcc_mask,
+		    if (firsttile && !TTMaskHasType(&cptr->drcc_mask,
 				TiGetRightType(BL(tile))) &&
 				!TTMaskHasType(&cptr->drcc_mask,
 				TiGetTopType(LB(tile))))
@@ -571,13 +603,13 @@ drcTile (tile, arg)
 		do {
 		    if (triggered)
 		    {
-			/* Widespacing hack---rule is triggered by MAXWIDTH	*/
-			/* algorithm, but we only want to look in the region	*/
-			/* of the maxwidth rectangle.  Otherwise, a violation	*/
-			/* is incorrectly flagged on thin wires that happen to	*/
-			/* share a common edge with the maxwidth region.	*/
+			/* For triggered rules, we want to look	at the	*/
+			/* clipped region found by the triggering rule	*/
 
-			errRect = mrd->rlist[--triggered];
+			if (mrd)
+			    errRect = mrd->rlist[--triggered];
+			else
+			    errRect = arg->dCD_rlist[--triggered];
 			errRect.r_ytop += cdist;
 			errRect.r_ybot -= cdist;
 			if (errRect.r_ytop > edgeTop) errRect.r_ytop = edgeTop;
@@ -650,6 +682,7 @@ drcTile (tile, arg)
 			 * by looking only at tpleft.
 			 */
 			errRect.r_xbot = edgeX - dist;
+			if (cptr->drcc_flags & DRC_OUTSIDE) errRect.r_xbot--;
 			if (LEFT(tpleft) <= errRect.r_xbot
 				&& BOTTOM(tpleft) <= errRect.r_ybot
 				&& TOP(tpleft) >= errRect.r_ytop
@@ -659,6 +692,7 @@ drcTile (tile, arg)
 			    continue;
 
 			errRect.r_xtop = edgeX;
+			if (cptr->drcc_flags & DRC_OUTSIDE) errRect.r_xtop -= dist;
 			arg->dCD_initial = tile;
 		    }
 		    else  /* FORWARD */
@@ -721,6 +755,7 @@ drcTile (tile, arg)
 			 * by looking only at tile.
 			 */
 			errRect.r_xtop = edgeX + dist;
+			if (cptr->drcc_flags & DRC_OUTSIDE) errRect.r_xtop++;
 			if (RIGHT(tile) >= errRect.r_xtop
 				&& BOTTOM(tile) <= errRect.r_ybot
 				&& TOP(tile) >= errRect.r_ytop
@@ -730,6 +765,7 @@ drcTile (tile, arg)
 			    continue;
 
 			errRect.r_xbot = edgeX;
+			if (cptr->drcc_flags & DRC_OUTSIDE) errRect.r_xbot += dist;
 			arg->dCD_initial= tpleft;
 		    }
 		    if (arg->dCD_radial)
@@ -740,14 +776,14 @@ drcTile (tile, arg)
 
 		    DRCstatSlow++;
 		    arg->dCD_cptr = cptr;
+		    arg->dCD_entries = 0;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
-
-		    result += DBSrPaintArea((Tile *) NULL,
+		    DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
 		} while (triggered);
 
-		if (result == 0)
+		if (arg->dCD_entries == 0)
 		{
 		    /* Trigger rule:  If rule check found errors,	*/
 		    /* do the next rule.  Otherwise, skip it.		*/
@@ -755,8 +791,11 @@ drcTile (tile, arg)
 		    if (trigpending)
 			cptr = cptr->drcc_next;
 		}
+		else
+		    triggered = arg->dCD_entries;
 	    }
 	    DRCstatEdges++;
+	    firsttile = FALSE;
         }
     }
 
@@ -804,6 +843,7 @@ checkbottom:
 	int edgeY = BOTTOM(tile);
 
 	/* Go right across bottom of tile */
+	firsttile = TRUE;
         for (tpbot = LB(tile); LEFT(tpbot) < right; tpbot = TR(tpbot))
         {
 	    /* Get the tile types to the top and bottom of the edge */
@@ -866,8 +906,10 @@ checkbottom:
 
 		    if (cptr->drcc_flags & DRC_REVERSE)
 			mrd = drcCanonicalMaxwidth(tpbot, GEO_SOUTH, arg, cptr);
-		    else
+		    else if (firsttile)
 			mrd = drcCanonicalMaxwidth(tile, GEO_NORTH, arg, cptr);
+		    else
+			mrd = NULL;
 		    if (!trigpending) cptr->drcc_dist--;
 		    if (trigpending)
 		    {
@@ -899,20 +941,24 @@ checkbottom:
 		    if (trigpending) cptr = cptr->drcc_next;
 		    continue;
 		}
+		else if (!triggered) mrd = NULL;
 
 		result = 0;
 		arg->dCD_radial = 0; 
 		do {
 		    if (triggered)
 		    {
-			errRect = mrd->rlist[--triggered];
+			/* For triggered rules, we want to look	at the	*/
+			/* clipped region found by the triggering rule	*/
+
+			if (mrd)
+			    errRect = mrd->rlist[--triggered];
+			else
+			    errRect = arg->dCD_rlist[--triggered];
 			errRect.r_xtop += cdist;
 			errRect.r_xbot -= cdist;
 			if (errRect.r_xtop > edgeRight) errRect.r_xtop = edgeRight;
 			if (errRect.r_xbot < edgeLeft) errRect.r_xbot = edgeLeft;
-
-			/* Temporary. . . we need to loop through all rects */
-			triggered = 0;
 		    }
 		    else
 		    {
@@ -960,6 +1006,7 @@ checkbottom:
 			 * a messy search by looking only at tpbot.
 			 */
 			errRect.r_ybot = edgeY - dist;
+			if (cptr->drcc_flags & DRC_OUTSIDE) errRect.r_ybot--;
 			if (BOTTOM(tpbot) <= errRect.r_ybot
 				&& LEFT(tpbot) <= errRect.r_xbot
 				&& RIGHT(tpbot) >= errRect.r_xtop
@@ -969,6 +1016,7 @@ checkbottom:
 			    continue;
 
 			errRect.r_ytop = edgeY;
+			if (cptr->drcc_flags & DRC_OUTSIDE) errRect.r_ytop -= dist;
 			arg->dCD_initial = tile;
 		    }
 		    else  /* FORWARD */
@@ -1011,6 +1059,7 @@ checkbottom:
 			 * a messy search by looking only at tile.
 			 */
 			errRect.r_ytop = edgeY + dist;
+			if (cptr->drcc_flags & DRC_OUTSIDE) errRect.r_ytop++;
 			if (TOP(tile) >= errRect.r_ytop
 				&& LEFT(tile) <= errRect.r_xbot
 				&& RIGHT(tile) >= errRect.r_xtop
@@ -1020,6 +1069,7 @@ checkbottom:
 			    continue;
 
 			errRect.r_ybot = edgeY;
+			if (cptr->drcc_flags & DRC_OUTSIDE) errRect.r_ybot += dist;
 			arg->dCD_initial = tpbot;
 		    }
 		    if (arg->dCD_radial)
@@ -1030,13 +1080,14 @@ checkbottom:
 
 		    DRCstatSlow++;
 		    arg->dCD_cptr = cptr;
+		    arg->dCD_entries = 0;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
-		    result += DBSrPaintArea((Tile *) NULL,
+		    DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
 		} while (triggered);
 
-		if (result == 0)
+		if (arg->dCD_entries == 0)
 		{
 		    /* Trigger rule:  If rule check found errors,	*/
 		    /* do the next rule.  Otherwise, skip it.	*/
@@ -1044,8 +1095,11 @@ checkbottom:
 		    if (trigpending)
 			cptr = cptr->drcc_next;
 		}
+		else
+		    triggered = arg->dCD_entries;
 	    }
 	    DRCstatEdges++;
+	    firsttile = FALSE;
         }
     }
     return (0);
