@@ -14,7 +14,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/ext2spice/ext2hier.c,v 1.2 2010/08/25 17:33:56 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/ext2spice/ext2hier.c,v 1.5 2010/12/16 18:59:03 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -98,7 +98,8 @@ GetHierNode(hc, name)
     EFNodeName *nn;
     Def *def = hc->hc_use->use_def;
 
-    he = HashFind(&def->def_nodes, EFHNToStr(name));
+    // he = HashFind(&def->def_nodes, EFHNToStr(name));
+    he = EFHNConcatLook(hc->hc_hierName, name, "node");
     if (he == NULL) return NULL;
     nn = (EFNodeName *) HashGetValue(he);
     if (nn == NULL) return NULL;
@@ -145,8 +146,10 @@ esOutputHierResistor(hc, dev, trans, term1, term2, has_model, l, w, dscale)
     /* term1=gate term2=source by the above code.	 */
     /* extracted units are Ohms; output is in Ohms 	 */
 
-    spcdevHierOutNode(hc, term1->dterm_node->efnode_name->efnn_hier, esSpiceF);
-    spcdevHierOutNode(hc, term2->dterm_node->efnode_name->efnn_hier, esSpiceF);
+    spcdevOutNode(hc->hc_hierName, term1->dterm_node->efnode_name->efnn_hier,
+		"res_top", esSpiceF);
+    spcdevOutNode(hc->hc_hierName, term2->dterm_node->efnode_name->efnn_hier,
+		"res_bot", esSpiceF);
 
     sdM = getCurDevMult();
 
@@ -170,15 +173,15 @@ esOutputHierResistor(hc, dev, trans, term1, term2, has_model, l, w, dscale)
 	if (esScale < 0) 
 	{
 	    fprintf(esSpiceF, " w=%d l=%d", w * scale, (l * scale) / dscale);
-	    if (sdM != 1.0)
-		fprintf(esSpiceF, " M=%g", sdM);
 	}
 	else
 	{
 	    fprintf(esSpiceF, " w=%gu l=%gu",
-		(float)w * scale * esScale * sdM,
+		(float)w * scale * esScale,
 		(float)((l * scale * esScale) / dscale));
 	}
+	if (sdM != 1.0)
+	    fprintf(esSpiceF, " M=%g", sdM);
     }
 }
 
@@ -198,6 +201,8 @@ subcktHierVisit(use, hierName, is_top)
     EFNode *snode;
     bool hasports = FALSE;
 
+    if (def->def_flags & DEF_NODEVICES) return 0;
+
     /* Avoid generating records for circuits that have no ports.	*/
     /* These are already absorbed into the parent.  All other		*/
     /* subcircuits have at least one port marked by the EF_PORT flag.	*/
@@ -205,7 +210,7 @@ subcktHierVisit(use, hierName, is_top)
     for (snode = (EFNode *) def->def_firstn.efnode_next;
 		snode != &def->def_firstn;
 		snode = (EFNode *) snode->efnode_next)
-	if (snode->efnode_flags & EF_PORT)
+	if (snode->efnode_flags & (EF_PORT | EF_SUBS_PORT))
 	{
 	    hasports = TRUE;
 	    break;
@@ -390,18 +395,21 @@ spcdevHierVisit(hc, dev, trans)
 	    /* BJT is "Qnnn collector emitter base model" 			*/
 	    /* extraction sets collector=subnode, emitter=gate, base=drain	*/
 
-	    spcdevHierOutNode(hc, subnode->efnode_name->efnn_hier, esSpiceF); 
-	    spcdevHierOutNode(hc, gate->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+	    spcdevOutNode(hc->hc_hierName, subnode->efnode_name->efnn_hier,
+			"collector", esSpiceF); 
+	    spcdevOutNode(hc->hc_hierName, gate->dterm_node->efnode_name->efnn_hier,
+			"emitter", esSpiceF);
 
 	    /* fix mixed up drain/source for bjts hace 2/2/99 */
 	    if (gate->dterm_node->efnode_name->efnn_hier ==
 			source->dterm_node->efnode_name->efnn_hier)
-		spcdevHierOutNode(hc, drain->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+		spcdevOutNode(hc->hc_hierName,
+			drain->dterm_node->efnode_name->efnn_hier,
+			"base", esSpiceF);
 	    else
-		spcdevHierOutNode(hc, source->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+		spcdevOutNode(hc->hc_hierName,
+			source->dterm_node->efnode_name->efnn_hier,
+			"base", esSpiceF);
 
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 	    break;
@@ -411,8 +419,9 @@ spcdevHierVisit(hc, dev, trans)
 	    /* Subcircuit is "Xnnn gate [source [drain [sub]]]"		*/
 	    /* Subcircuit .subckt record must be ordered to match!	*/
 
-	    spcdevHierOutNode(hc, gate->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			gate->dterm_node->efnode_name->efnn_hier,
+			"subckt", esSpiceF);
 
 	    /* Drop through to below (no break statement) */
 
@@ -422,11 +431,13 @@ spcdevHierVisit(hc, dev, trans)
 	    /* only and is not output.					*/
 
 	    if (dev->dev_nterm > 1)
-		spcdevHierOutNode(hc, source->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+		spcdevOutNode(hc->hc_hierName,
+			source->dterm_node->efnode_name->efnn_hier,
+			"subckt", esSpiceF);
 	    if (dev->dev_nterm > 2)
-		spcdevHierOutNode(hc, drain->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+		spcdevOutNode(hc->hc_hierName,
+			drain->dterm_node->efnode_name->efnn_hier,
+			"subckt", esSpiceF);
 
 	    /* The following only applies to DEV_SUBCKT*, which may define as	*/
 	    /* many terminal types as it wants.					*/
@@ -434,8 +445,9 @@ spcdevHierVisit(hc, dev, trans)
 	    for (i = 4; i < dev->dev_nterm; i++)
 	    {
 		drain = &dev->dev_terms[i - 1];
-		spcdevHierOutNode(hc, drain->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+		spcdevOutNode(hc->hc_hierName,
+			drain->dterm_node->efnode_name->efnn_hier,
+			"subckt", esSpiceF);
 	    }
 
 	    /* Get the device parameters now, and check if the substrate is	*/
@@ -449,7 +461,7 @@ spcdevHierVisit(hc, dev, trans)
 	    if ((pptr == NULL) && subnode)
 	    {
 		fprintf(esSpiceF, " ");
-		subnodeFlat = spcdevHierSubstrate(hc,
+		subnodeFlat = spcdevSubstrate(hc->hc_hierName,
 			subnode->efnode_name->efnn_hier,
 			dev->dev_type, esSpiceF);
 	    }
@@ -470,15 +482,15 @@ spcdevHierVisit(hc, dev, trans)
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%d", dev->dev_area * scale * scale);
 			else
-			    fprintf(esSpiceF, "%gu", dev->dev_area * scale * scale
-					* esScale * esScale * sdM);
+			    fprintf(esSpiceF, "%gp", dev->dev_area * scale * scale
+					* esScale * esScale);
 			break;
 		    case 'p':
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%d", dev->dev_perim * scale);
 			else
 			    fprintf(esSpiceF, "%gu", dev->dev_perim * scale
-					* esScale * sdM);
+					* esScale);
 			break;
 		    case 'l':
 			if (esScale < 0)
@@ -490,10 +502,10 @@ spcdevHierVisit(hc, dev, trans)
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%d", w * scale);
 			else
-			    fprintf(esSpiceF, "%gu", w * scale * esScale * sdM);
+			    fprintf(esSpiceF, "%gu", w * scale * esScale);
 			break;
 		    case 's':
-			subnodeFlat = spcdevHierSubstrate(hc,
+			subnodeFlat = spcdevSubstrate(hc->hc_hierName,
 				subnode->efnode_name->efnn_hier,
 				dev->dev_type, esSpiceF);
 			break;
@@ -512,17 +524,15 @@ spcdevHierVisit(hc, dev, trans)
 					* esScale);
 			break;
 		    case 'r':
-			fprintf(esSpiceF, "%f", (double)(dev->dev_res) /
-					(double)sdM);
+			fprintf(esSpiceF, "%f", (double)(dev->dev_res));
 			break;
 		    case 'c':
-			fprintf(esSpiceF, "%ff", (double)sdM *
-					(double)(dev->dev_cap));
+			fprintf(esSpiceF, "%ff", (double)(dev->dev_cap));
 			break;
 		}
 		plist = plist->parm_next;
 	    }
-	    if ((esScale < 0) && (sdM != 1.0))
+	    if (sdM != 1.0)
 		fprintf(esSpiceF, " M=%g", sdM);
 	    break;
 
@@ -565,10 +575,12 @@ spcdevHierVisit(hc, dev, trans)
 
 	    /* Diode is "Dnnn top bottom model"	*/
 
-	    spcdevHierOutNode(hc, gate->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
-	    spcdevHierOutNode(hc, source->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			gate->dterm_node->efnode_name->efnn_hier,
+			"diode_top", esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			source->dterm_node->efnode_name->efnn_hier,
+			"diode_bot", esSpiceF);
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 	    break;
 
@@ -579,10 +591,12 @@ spcdevHierVisit(hc, dev, trans)
 	    /* extraction sets top=gate bottom=source	*/
 	    /* extracted units are fF; output is in fF */
 
-	    spcdevHierOutNode(hc, gate->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
-	    spcdevHierOutNode(hc, source->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			gate->dterm_node->efnode_name->efnn_hier,
+			"cap_top", esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			source->dterm_node->efnode_name->efnn_hier,
+			"cap_bot", esSpiceF);
 
 	    sdM = getCurDevMult();
 
@@ -606,16 +620,15 @@ spcdevHierVisit(hc, dev, trans)
 		if (esScale < 0)
 		{
 		    fprintf(esSpiceF, " w=%d l=%d", w*scale, l*scale);
-		    if (sdM != 1.0)
-			fprintf(esSpiceF, " M=%g", sdM);
 		}
 		else
 		{
 		    fprintf(esSpiceF, " w=%gu l=%gu",
-			(float)w * scale * esScale * sdM,
+			(float)w * scale * esScale,
 			(float)l * scale * esScale);
-		    sdM = 1.0;
 		}
+		if (sdM != 1.0)
+		    fprintf(esSpiceF, " M=%g", sdM);
 	    }
 	    break;
 
@@ -626,16 +639,19 @@ spcdevHierVisit(hc, dev, trans)
 
 	    /* MOSFET is "Mnnn drain gate source [L=x W=x [attributes]]" */
 
-	    spcdevHierOutNode(hc, drain->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
-	    spcdevHierOutNode(hc, gate->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
-	    spcdevHierOutNode(hc, source->dterm_node->efnode_name->efnn_hier,
-			esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			drain->dterm_node->efnode_name->efnn_hier,
+			"drain", esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			gate->dterm_node->efnode_name->efnn_hier,
+			"gate", esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			source->dterm_node->efnode_name->efnn_hier,
+			"source", esSpiceF);
 	    if (subnode)
 	    {
 		fprintf(esSpiceF, " ");
-		subnodeFlat = spcdevHierSubstrate(hc,
+		subnodeFlat = spcdevSubstrate(hc->hc_hierName,
 			subnode->efnode_name->efnn_hier,
 			dev->dev_type, esSpiceF);
 	    }
@@ -657,16 +673,15 @@ spcdevHierVisit(hc, dev, trans)
 	    if (esScale < 0)
 	    {
 		fprintf(esSpiceF, " w=%d l=%d", w*scale, l*scale);
-		if (sdM != 1.0)
-		    fprintf(esSpiceF, " M=%g", sdM);
 	    }
 	    else
 	    {
 		fprintf(esSpiceF, " w=%gu l=%gu",
-			(float)w * scale * esScale * sdM,
+			(float)w * scale * esScale,
 			(float)l * scale * esScale);
-		sdM = 1.0;
 	    }
+	    if (sdM != 1.0)
+		fprintf(esSpiceF, " M=%g", sdM);
 
 	    /*
 	     * Check controlling attributes and output area and perimeter. 
@@ -720,112 +735,90 @@ spcdevHierVisit(hc, dev, trans)
 /*
  * ----------------------------------------------------------------------------
  *
- * spcdevHierSubstrate -
+ * spcdevHierMergeVisit --
  *
- * Output the node name of the substrate of a dev. If the name is the 
- * same as the default dont go looking for it just output the default
- * (trimmed appropriately). Otherwise look it up ..
- *
- * Results:
- *   NULL if not found or the default substrate or the node pointer
- *   otherwise (might be reused to output area and perimeter of
- *   the substrate).
- *
- * Side effects:
- *  Might allocate the nodeClient for the node through nodeSpiceHierName.
- *
- * ----------------------------------------------------------------------------
- */
-EFNode *spcdevHierSubstrate(hc, name, type, outf)
-    HierContext *hc;
-    HierName *name;
-    int type;
-    FILE *outf;
-{
-    HashEntry *he;
-    EFNodeName *nn;
-    char *suf ;
-    int  l ;
-    Def *def = hc->hc_use->use_def;
-
-    suf = EFHNToStr(name);
-    if (fetInfo[type].defSubs && strcasecmp(suf,fetInfo[type].defSubs) == 0) {
-	if ( outf ) {
-    	   l = strlen(suf) - 1;
-	   if (  ( EFTrimFlags & EF_TRIMGLOB ) && suf[l] =='!' ||
-	         ( EFTrimFlags & EF_TRIMLOCAL ) && suf[l] == '#'  )
-	        suf[l] = '\0' ;
-	   fprintf(outf, "%s", suf);
-	}
-	return NULL;
-    }
-    else {
-        he = HashFind(&def->def_nodes, EFHNToStr(name));
-    	if (he == NULL)
-    	{
-		if (outf) 
-		   fprintf(outf, "errGnd!");
-		return NULL;
-    	}
-    	/* Canonical name */
-    	nn = (EFNodeName *) HashGetValue(he);
-	if (outf) 
-	{
-	    if (nn == NULL)
-		fprintf(outf, "<invalid node>");
-	    else
-		fprintf(outf,
-			nodeSpiceHierName(hc, nn->efnn_node->efnode_name->efnn_hier));
-	}
-	if (nn == NULL)
-	    return NULL;
-	else
-            return nn->efnn_node;
-   }
-}
-
-
-/*
- * ----------------------------------------------------------------------------
- *
- * spcdevHierOutNode --
- *
- * Output the name of the node whose name from the end of suffix down
- * to the leaves is 'name'.
- *
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Writes to the file 'outf'.
- *	Sets the efnode_client field as described above.
+ * First pass visit to devices to determine if they can be merged with
+ * any previously visited device.
  *
  * ----------------------------------------------------------------------------
  */
 
 int
-spcdevHierOutNode(hc, name, outf)
+spcdevHierMergeVisit(hc, dev, trans)
     HierContext *hc;
-    HierName *name;
-    FILE *outf;
+    Dev *dev;		/* Dev being output */
+    Transform *trans;	/* Coordinate transform for output */
 {
-    HashEntry *he;
-    EFNodeName *nn;
-    Def *def = hc->hc_use->use_def;
+    DevTerm *gate, *source, *drain;
+    EFNode *subnode, *snode, *dnode, *gnode;
+    Rect r;
+    int scale, pmode, l, w;
+    devMerge *fp, *cfp;
+    float m;
 
-    he = HashFind(&def->def_nodes, EFHNToStr(name));
-    if (he == NULL)
+    /* If no terminals, or only a gate, can't do much of anything */
+    if (dev->dev_nterm < 2) return 0;
+
+    gate = &dev->dev_terms[0];
+    source = drain = &dev->dev_terms[1];
+    if (dev->dev_nterm >= 3)
+	drain = &dev->dev_terms[2];
+
+    gnode = GetHierNode(hc, gate->dterm_node->efnode_name->efnn_hier);
+    snode = GetHierNode(hc, source->dterm_node->efnode_name->efnn_hier);
+    dnode = GetHierNode(hc, drain->dterm_node->efnode_name->efnn_hier);
+    subnode = dev->dev_subsnode;
+
+    GeoTransRect(trans, &dev->dev_rect, &r);
+    scale = GeoScale(trans);
+
+    EFGetLengthAndWidth(dev, &l, &w);
+
+    fp = mkDevMerge(l * scale, w * scale, gnode, snode, dnode, subnode,
+		hc->hc_hierName, dev);
+		
+    for (cfp = devMergeList; cfp != NULL; cfp = cfp->next)
     {
-	fprintf(outf, " errGnd!");
-	return;
+	if ((pmode = parallelDevs(fp, cfp)) != NOT_PARALLEL)
+	{
+	    /* To-do:  add back source, drain attribute check */
+
+	    switch(dev->dev_class)
+	    {
+		case DEV_MOSFET:
+		case DEV_ASYMMETRIC:
+		case DEV_FET:
+		    m = esFMult[cfp->esFMIndex] + ((float)fp->w / (float)cfp->w);
+		    break;
+		case DEV_RES:
+		    if (fp->dev->dev_type == esNoModelType)
+			m = esFMult[cfp->esFMIndex] + (fp->dev->dev_res
+				/ cfp->dev->dev_res);
+		    else
+			m = esFMult[cfp->esFMIndex] + ((float)fp->l / (float)cfp->l);
+		    break;
+		case DEV_CAP:
+		    if (fp->dev->dev_type == esNoModelType)
+			m = esFMult[cfp->esFMIndex] + (fp->dev->dev_cap
+				/ cfp->dev->dev_cap);
+		    else
+			m = esFMult[cfp->esFMIndex] +
+				(((float)fp->l * (float)fp->w)
+				/ ((float)cfp->l * (float)cfp->w));
+		    break;
+	    }
+	    setDevMult(fp->esFMIndex, DEV_KILLED);
+	    setDevMult(cfp->esFMIndex, m);
+	    esDevsMerged++;
+	    freeMagic(fp);
+	    return 0;
+	}
     }
-    nn = (EFNodeName *) HashGetValue(he);
-    if (nn == NULL)
-	fprintf(outf, " <invalid node>");
-    else
-        fprintf(outf, " %s",
-		nodeSpiceHierName(hc, nn->efnn_node->efnode_name->efnn_hier));
+
+    /* No devices are parallel to this one (yet) */
+    fp->next = devMergeList;
+    devMergeList = fp;
+    return 0;
 }
 
 
@@ -1023,7 +1016,7 @@ devMergeHierVisit(hc, dev, trans)
     snode = GetHierNode(hc, source->dterm_node->efnode_name->efnn_hier);
     dnode = GetHierNode(hc, drain->dterm_node->efnode_name->efnn_hier);
     if (dev->dev_subsnode)
-	subnode = spcdevHierSubstrate(hc,
+	subnode = spcdevSubstrate(hc->hc_hierName,
 			dev->dev_subsnode->efnode_name->efnn_hier, 
 			dev->dev_type, NULL);
     else
@@ -1183,11 +1176,11 @@ esMakePorts(hc, cdata)
     ClientData cdata;
 {
     Connection *conn;
-    Def *def = hc->hc_use->use_def, *portdef;
+    Def *def = hc->hc_use->use_def, *portdef, *updef;
     Use *use;
     HashEntry *he;
     EFNodeName *nn;
-    char *name, *portname, *tptr, *locname;
+    char *name, *portname, *tptr, *aptr, *locname;
     int j;
 
     if (def->def_uses == NULL) return 0;	/* Bottom of hierarchy */
@@ -1202,12 +1195,22 @@ esMakePorts(hc, cdata)
 		continue;
 
 	    portname = name;
+	    updef = def;
+
 	    while (tptr != NULL)
 	    {
+		/* Ignore array information for the purpose of tracing	*/	
+		/* the cell definition hierarchy.			*/
+
+		aptr = strchr(portname, '[');
+		if (aptr == NULL)
+		    *tptr = '\0';
+		else
+		    *aptr = '\0';
+
 		// Find the cell for the instance
-		*tptr = '\0';
 		portdef = NULL;
-		for (use = def->def_uses; use; use = use->use_next)
+		for (use = updef->def_uses; use; use = use->use_next)
 		{
 		    if (!strcmp(use->use_id, portname))
 		    {
@@ -1215,7 +1218,10 @@ esMakePorts(hc, cdata)
 			break;
 		    }
 		}
-		*tptr = '/';
+		if (aptr == NULL)
+		    *tptr = '/';
+		else
+		    *aptr = '[';
 		portname = tptr + 1;
 
 		// Find the net of portname in the subcell and
@@ -1240,11 +1246,12 @@ esMakePorts(hc, cdata)
 
 		if ((tptr = strchr(portname, '/')) == NULL)
 		    break;
+
+		updef = portdef;
 	    }
 	    // Diagnostic
 	    // TxPrintf("Connection in %s to net %s (%s)\n", def->def_name,
 	    //		name, portname);
-
 	}
     }
     return 0;
@@ -1282,13 +1289,22 @@ esHierVisit(hc, cdata)
 	for (snode = (EFNode *) def->def_firstn.efnode_next;
 		snode != &def->def_firstn;
 		snode = (EFNode *) snode->efnode_next)
-	    snode->efnode_flags &= ~EF_PORT;
+	    snode->efnode_flags &= ~(EF_PORT | EF_SUBS_PORT);
 	return 0;
     } 
 
     /* Flatten this definition only */
     hcf = EFFlatBuildOneLevel(hc->hc_use->use_def);
 
+    /* If definition has been marked as having no devices, then this	*/
+    /* def is not to be output.						*/
+
+    if (hc->hc_use->use_def->def_flags & DEF_NODEVICES)
+    {
+	EFFlatDone();
+	return 0;
+    }
+
     /* Generate subcircuit header */
     if (def != topdef)
 	topVisit(hc->hc_use->use_def);
@@ -1298,6 +1314,19 @@ esHierVisit(hc, cdata)
     /* Output subcircuit calls */
     EFHierVisitSubcircuits(hcf, subcktHierVisit, (ClientData)NULL);
 
+    /* Merge devices */
+    if (esMergeDevsA || esMergeDevsC)
+    {
+	devMerge *p;
+
+	EFHierVisitDevs(hcf, spcdevHierMergeVisit, (ClientData)NULL);
+	TxPrintf("Devs merged: %d\n", esDevsMerged);
+	esFMIndex = 0;
+	for (p = devMergeList; p != NULL; p = p->next)
+	    freeMagic(p);
+	devMergeList = NULL;
+    }
+
     /* Output devices */
     EFHierVisitDevs(hcf, spcdevHierVisit, (ClientData)NULL);
 
@@ -1309,9 +1338,18 @@ esHierVisit(hc, cdata)
 
     if (def != topdef)
 	fprintf(esSpiceF, ".ends\n\n");
-     else
+    else
 	fprintf(esSpiceF, ".end\n\n");
 
+    /* Reset device/node/subcircuit instance counts */
+
+    esCapNum  = 0;
+    esDevNum = 1000;
+    esResNum = 0;
+    esDiodeNum = 0;
+    esSbckNum = 0;
+    esNodeNum = 10;
+
     EFFlatDone();
     return 0;
 }
