@@ -100,25 +100,11 @@ int	 esFMIndex = 0;          /* current index to it */
 int	 esFMSize = FMULT_SIZE ; /* its current size (growable) */
 int	 esDevsMerged;
 
-/* macro to add a dev's multiplier to the table and grow it if necessary */
-#define addDevMult(f) \
-{  \
-	if ( esFMult == NULL ) { \
-	  esFMult = (float *) mallocMagic((unsigned) (esFMSize*sizeof(float)));  \
-	} else if ( esFMIndex >= esFMSize ) {  \
-	  int i;  \
-	  float *op = esFMult ;  \
-	  esFMult = (float *) mallocMagic((unsigned) ((esFMSize = esFMSize*2)*sizeof(float))); \
-	  for ( i = 0 ; i < esFMSize/2 ; i++ ) esFMult[i] = op[i]; \
-	  if (op) freeMagic(op); \
-	}  \
-	esFMult[esFMIndex++] = (float)(f); \
-}
-
 devMerge *devMergeList = NULL ;
 
 #ifdef MAGIC_WRAPPER
 
+#ifdef EXT2SPICE_AUTO
 /*
  * ----------------------------------------------------------------------------
  *
@@ -153,6 +139,7 @@ Exttospice_Init(interp)
     Tcl_PkgProvide(interp, "Exttospice", MAGIC_VERSION);
     return TCL_OK;
 }
+#endif /* EXT2SPICE_AUTO */
 
 /*
  * ----------------------------------------------------------------------------
@@ -680,6 +667,13 @@ runexttospice:
 	if ((esDoSubckt == TRUE) || (locDoSubckt == TRUE))
 	    topVisit(efFlatRootDef);
 
+	/* When generating subcircuits, remove the subcircuit	*/
+	/* flag from the top level cell.  Other than being	*/
+	/* used to generate the subcircuit wrapper, it should	*/
+	/* not prevent descending into its own hierarchy.	*/
+
+	efFlatRootDef->def_flags &= ~(DEF_SUBCIRCUIT);
+
 	/* If we don't want to write subcircuit calls, remove	*/
 	/* the subcircuit flag from all cells at this time.	*/
 
@@ -690,16 +684,14 @@ runexttospice:
 
 	if (esMergeDevsA || esMergeDevsC)
 	{
+	    devMerge *p;
+
 	    EFVisitDevs(devMergeVisit, (ClientData) NULL);
 	    TxPrintf("Devs merged: %d\n", esDevsMerged);
 	    esFMIndex = 0;
-	    {
-		devMerge *p;
-
-		for (p = devMergeList; p != NULL; p = p->next)
-		    freeMagic(p);
-		devMergeList = NULL;
-	    }
+	    for (p = devMergeList; p != NULL; p = p->next)
+		freeMagic(p);
+	    devMergeList = NULL;
 	}
 	else if (esDistrJunct)
      	    EFVisitDevs(devDistJunctVisit, (ClientData) NULL);
@@ -1044,7 +1036,7 @@ usage:
 /*
  * ----------------------------------------------------------------------------
  *
- * GetNode --
+ * SpiceGetNode --
  *
  * function to find a node given its hierarchical prefix and suffix
  *
@@ -1054,7 +1046,7 @@ usage:
  * ----------------------------------------------------------------------------
  */
 EFNode *
-GetNode(prefix, suffix)
+SpiceGetNode(prefix, suffix)
 HierName *prefix;
 HierName *suffix;
 {
@@ -1136,7 +1128,8 @@ subcktVisit(use, hierName, is_top)
     EFNode *snode;
     Def *def = use->use_def;
     EFNodeName *nodeName;
-    int portorder, portmax;
+    int portorder, portmax, imp_max;
+    char stmp[MAX_STR_SIZE];
 
     if (is_top == TRUE) return 0;	/* Ignore the top-level cell */
 
@@ -1149,19 +1142,7 @@ subcktVisit(use, hierName, is_top)
     /* Note that the ports of the subcircuit will not necessarily be	*/
     /* ALL the entries in the hash table, so we have to check.		*/
 
-    portmax = -1;
-    for (snode = (EFNode *) def->def_firstn.efnode_next;
-		snode != &def->def_firstn;
-		snode = (EFNode *) snode->efnode_next)
-    {
-	if (!(snode->efnode_flags & EF_PORT)) continue;
-	for (nodeName = snode->efnode_name; nodeName != NULL; nodeName =
-			nodeName->efnn_next)
-	{
-	    portorder = nodeName->efnn_port;
-	    if (portorder > portmax) portmax = portorder;
-	}
-    }
+    portmax = EFGetPortMax(def, &imp_max);
 
     if (portmax < 0)
     {
@@ -1172,12 +1153,36 @@ subcktVisit(use, hierName, is_top)
         for (snode = (EFNode *) def->def_firstn.efnode_next;
 		snode != &def->def_firstn;
 		snode = (EFNode *) snode->efnode_next)
+	{
 	    if (snode->efnode_flags & EF_PORT)
 		for (nodeName = snode->efnode_name; nodeName != NULL;
 			nodeName = nodeName->efnn_next)
 		    if (nodeName->efnn_port >= 0)
+		    {
+			portmax++;
 			spcdevOutNode(hierName, nodeName->efnn_hier,
 					"subcircuit", esSpiceF); 
+		    }
+	}
+
+	/* Look for all implicit substrate connections that are	*/
+	/* declared as local node names, and put them last.	*/
+
+	for (snode = (EFNode *) def->def_firstn.efnode_next;
+		snode != &def->def_firstn;
+		snode = (EFNode *) snode->efnode_next)
+	{
+	    if (snode->efnode_flags & EF_SUBS_PORT)
+	    {
+		nodeName = snode->efnode_name;
+		if (nodeName->efnn_port < 0)
+	            nodeName->efnn_port = ++portmax;
+
+		/* This is not a hierarchical name or node! */
+		EFHNSprintf(stmp, nodeName->efnn_hier);
+		fprintf(esSpiceF, " %s", stmp);
+	    }
+	}
     }
     else
     {
@@ -1200,7 +1205,6 @@ subcktVisit(use, hierName, is_top)
 		    {
 			spcdevOutNode(hierName, nodeName->efnn_hier,
 					"subcircuit", esSpiceF); 
-
 			break;
 		    }
 		}
@@ -1208,6 +1212,28 @@ subcktVisit(use, hierName, is_top)
 	    }
 	    portorder++;
 	}
+
+	/* Look for all implicit substrate connections that are	*/
+	/* declared as local node names, and put them last.	*/
+
+	portorder = portmax;
+	while (portorder <= imp_max)
+	{
+	    for (snode = (EFNode *) def->def_firstn.efnode_next;
+			snode != &def->def_firstn;
+			snode = (EFNode *) snode->efnode_next)
+	    {
+		if (!(snode->efnode_flags & EF_SUBS_PORT)) continue;
+		nodeName = snode->efnode_name;
+		if (nodeName->efnn_port == portorder)
+		{
+		    /* This is not a hierarchical name or node! */
+		    EFHNSprintf(stmp, nodeName->efnn_hier);
+		    fprintf(esSpiceF, " %s", stmp);
+		}
+	    }
+	    portorder++;
+	}
     }
 
     fprintf(esSpiceF, " %s\n", def->def_name);	/* subcircuit model name */
@@ -1297,7 +1323,7 @@ topVisit(def)
 	    if (portorder > portmax) portmax = portorder;
 	}
     }
-    
+
     if (portmax < 0)
     {
 	/* No port order declared; print them and number them	*/
@@ -1314,6 +1340,7 @@ topVisit(def)
 	    snode = sname->efnn_node;
 
 	    if (snode->efnode_flags & EF_PORT)
+	    {
 		if (snode->efnode_name->efnn_port < 0)
 		{
 		    // fprintf(esSpiceF, " %s", he->h_key.h_name);
@@ -1322,6 +1349,7 @@ topVisit(def)
 				nodeSpiceName(snode->efnode_name->efnn_hier));
 		    snode->efnode_name->efnn_port = portorder++;
 		}
+	    }
 	}
     }
     else
@@ -1341,6 +1369,7 @@ topVisit(def)
 		snode = sname->efnn_node;
 
 		if (!(snode->efnode_flags & EF_PORT)) continue;
+
 		for (nodeName = sname; nodeName != NULL; nodeName = nodeName->efnn_next)
 		{
 		    portidx = nodeName->efnn_port;
@@ -1359,6 +1388,30 @@ topVisit(def)
 	    portorder++;
 	}
     }
+
+    /* Add all implicitly-defined local substrate node names */
+
+    HashStartSearch(&hs);
+    while (he = HashNext(&def->def_nodes, &hs))
+    {
+	sname = (EFNodeName *) HashGetValue(he);
+	if (sname == NULL) continue;
+	snode = sname->efnn_node;
+
+	if (snode->efnode_flags & EF_SUBS_PORT)
+	{
+	    if (snode->efnode_name->efnn_port < 0)
+	    {
+		char stmp[MAX_STR_SIZE];
+
+		/* This is not a hierarchical name or node! */
+		EFHNSprintf(stmp, snode->efnode_name->efnn_hier);
+		fprintf(esSpiceF, " %s", stmp);
+		snode->efnode_name->efnn_port = portorder++;
+	    }
+	}
+    }
+
     fprintf(esSpiceF, "\n");
 }
 
@@ -1427,17 +1480,14 @@ esOutputResistor(dev, hierName, trans, term1, term2, has_model, l, w, dscale)
 	scale = GeoScale(trans);
 
 	if (esScale < 0) 
-	{
 	    fprintf(esSpiceF, " w=%d l=%d", w * scale, (l * scale) / dscale);
-	    if (sdM != 1.0)
-		fprintf(esSpiceF, " M=%g", sdM);
-	}
 	else
-	{
 	    fprintf(esSpiceF, " w=%gu l=%gu",
-		(float)w * scale * esScale * sdM,
+		(float)w * scale * esScale,
 		(float)((l * scale * esScale) / dscale));
-	}
+
+	if (sdM != 1.0)
+	    fprintf(esSpiceF, " M=%g", sdM);
     }
 }
 
@@ -1512,6 +1562,23 @@ spcdevVisit(dev, hierName, trans)
 	drain = &dev->dev_terms[2];
     subnode = dev->dev_subsnode;
 
+    /* Check for minimum number of terminals. */
+
+    switch(dev->dev_class)
+    {
+	case DEV_SUBCKT:
+	case DEV_RSUBCKT:
+	    break;
+	default:
+	    if (dev->dev_nterm < 2)
+	    {
+		TxError("Device other than subcircuit has only "
+			"one terminal\n");
+		return 0;
+	    }
+	    break;
+    }
+
     /* Original hack for BiCMOS, Tim 10/4/97, is deprecated.	*/
     /* Use of "device bjt" preferred to "fet" with model="npn".	*/
 
@@ -1525,6 +1592,8 @@ spcdevVisit(dev, hierName, trans)
     {
 	case DEV_RES:
 	case DEV_CAP:
+	    if (dev->dev_nterm < 1) 
+		return 0;
 	    if (dev->dev_type == esNoModelType)
 		has_model = FALSE;
 	    break;
@@ -1693,15 +1762,15 @@ spcdevVisit(dev, hierName, trans)
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%d", dev->dev_area * scale * scale);
 			else
-			    fprintf(esSpiceF, "%gu", dev->dev_area * scale * scale
-					* esScale * esScale * sdM);
+			    fprintf(esSpiceF, "%gp", dev->dev_area * scale * scale
+					* esScale * esScale);
 			break;
 		    case 'p':
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%d", dev->dev_perim * scale);
 			else
 			    fprintf(esSpiceF, "%gu", dev->dev_perim * scale
-					* esScale * sdM);
+					* esScale);
 			break;
 		    case 'l':
 			if (esScale < 0)
@@ -1713,7 +1782,7 @@ spcdevVisit(dev, hierName, trans)
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%d", w * scale);
 			else
-			    fprintf(esSpiceF, "%gu", w * scale * esScale * sdM);
+			    fprintf(esSpiceF, "%gu", w * scale * esScale);
 			break;
 		    case 's':
 			subnodeFlat = spcdevSubstrate(hierName,
@@ -1735,17 +1804,15 @@ spcdevVisit(dev, hierName, trans)
 					* esScale);
 			break;
 		    case 'r':
-			fprintf(esSpiceF, "%f", (double)(dev->dev_res) /
-					(double)sdM);
+			fprintf(esSpiceF, "%f", (double)(dev->dev_res));
 			break;
 		    case 'c':
-			fprintf(esSpiceF, "%ff", (double)sdM *
-					(double)(dev->dev_cap));
+			fprintf(esSpiceF, "%ff", (double)(dev->dev_cap));
 			break;
 		}
 		plist = plist->parm_next;
 	    }
-	    if ((esScale < 0) && (sdM != 1.0))
+	    if (sdM != 1.0)
 		fprintf(esSpiceF, " M=%g", sdM);
 	    break;
 
@@ -1825,18 +1892,14 @@ spcdevVisit(dev, hierName, trans)
 		scale = GeoScale(trans);
 
 		if (esScale < 0)
-		{
 		    fprintf(esSpiceF, " w=%d l=%d", w*scale, l*scale);
-		    if (sdM != 1.0)
-			fprintf(esSpiceF, " M=%g", sdM);
-		}
 		else
-		{
 		    fprintf(esSpiceF, " w=%gu l=%gu",
-			(float)w * scale * esScale * sdM,
+			(float)w * scale * esScale,
 			(float)l * scale * esScale);
-		    sdM = 1.0;
-		}
+
+		if (sdM != 1.0)
+		    fprintf(esSpiceF, " M=%g", sdM);
 	    }
 	    break;
 
@@ -1875,18 +1938,14 @@ spcdevVisit(dev, hierName, trans)
 	    sdM = getCurDevMult();
 
 	    if (esScale < 0)
-	    {
 		fprintf(esSpiceF, " w=%d l=%d", w*scale, l*scale);
-		if (sdM != 1.0)
-		    fprintf(esSpiceF, " M=%g", sdM);
-	    }
 	    else
-	    {
 		fprintf(esSpiceF, " w=%gu l=%gu",
-			(float)w * scale * esScale * sdM,
+			(float)w * scale * esScale,
 			(float)l * scale * esScale);
-		sdM = 1.0;
-	    }
+
+	    if (sdM != 1.0)
+		fprintf(esSpiceF, " M=%g", sdM);
 
 	    /*
 	     * Check controlling attributes and output area and perimeter. 
@@ -1902,7 +1961,7 @@ spcdevVisit(dev, hierName, trans)
 			scale, "d", sdM, esSpiceF);
 	    else
 	    {
-		dnode = GetNode(hierName, drain->dterm_node->efnode_name->efnn_hier);
+		dnode = SpiceGetNode(hierName, drain->dterm_node->efnode_name->efnn_hier);
         	spcnAP(dnode, fetInfo[dev->dev_type].resClassSD, scale,
 			"d", sdM, esSpiceF, w);
 	    }
@@ -1910,7 +1969,7 @@ spcdevVisit(dev, hierName, trans)
 		spcnAPHier(source, hierName, fetInfo[dev->dev_type].resClassSD,
 			scale, "s", sdM, esSpiceF);
 	    else {
-		snode= GetNode(hierName, source->dterm_node->efnode_name->efnn_hier);
+		snode= SpiceGetNode(hierName, source->dterm_node->efnode_name->efnn_hier);
 		spcnAP(snode, fetInfo[dev->dev_type].resClassSD, scale,
 			"s", sdM, esSpiceF, w);
 	    }
@@ -2771,14 +2830,17 @@ devMergeVisit(dev, hierName, trans)
     }
 
     gate = &dev->dev_terms[0];
-    source = drain = &dev->dev_terms[1];
+    if (dev->dev_nterm >= 2)
+	source = drain = &dev->dev_terms[1];
     if (dev->dev_nterm >= 3)
 	drain = &dev->dev_terms[2];
 
-
-    gnode = GetNode(hierName, gate->dterm_node->efnode_name->efnn_hier);
-    snode = GetNode(hierName, source->dterm_node->efnode_name->efnn_hier);
-    dnode = GetNode(hierName, drain->dterm_node->efnode_name->efnn_hier);
+    gnode = SpiceGetNode(hierName, gate->dterm_node->efnode_name->efnn_hier);
+    if (dev->dev_nterm >= 2)
+    {
+	snode = SpiceGetNode(hierName, source->dterm_node->efnode_name->efnn_hier);
+	dnode = SpiceGetNode(hierName, drain->dterm_node->efnode_name->efnn_hier);
+    }
     if (dev->dev_subsnode)
 	subnode = spcdevSubstrate(hierName,
 			dev->dev_subsnode->efnode_name->efnn_hier, 
@@ -2966,7 +3028,7 @@ devDistJunctVisit(dev, hierName, trans)
 
     for (i = 1; i<dev->dev_nterm; i++)
     {
-	n = GetNode(hierName,
+	n = SpiceGetNode(hierName,
 		dev->dev_terms[i].dterm_node->efnode_name->efnn_hier);
 	update_w(fetInfo[dev->dev_type].resClassSD, w, n);
     }
