@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFflat.c,v 1.2 2010/08/25 17:33:56 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFflat.c,v 1.5 2010/12/16 18:59:03 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -160,6 +160,8 @@ EFFlatBuildOneLevel(def)
     Def *def;		/* root def being flattened */
 {
     int usecount;
+    Use *use;
+    int efFlatNodesDeviceless();	/* Forward declaration */
 
     efFlatRootDef = def;
 
@@ -195,14 +197,21 @@ EFFlatBuildOneLevel(def)
 
     /* Expand all subcells that contain connectivity information but	*/
     /* no active devices (including those in subcells).			*/
-    efFlatNodesDeviceless(&efFlatContext, &usecount);
 
-    /* Record all local nodes, if they have not already been counted */
-    if (usecount == 0)
-    {
-	efAddNodes(&efFlatContext, FALSE);
-	efAddConns(&efFlatContext);
-    }
+    for (use = efFlatRootUse.use_def->def_uses; use; use = use->use_next)
+	usecount++;
+
+    /* Recursively flatten uses that have no active devices */
+    if (usecount > 0)
+	efHierSrUses(&efFlatContext, efFlatNodesDeviceless, (ClientData)&usecount);
+
+    if ((usecount == 0) && (efFlatRootUse.use_def->def_devs == NULL))
+	efFlatRootUse.use_def->def_flags |= DEF_NODEVICES;
+
+    /* Record all local nodes */
+    efAddNodes(&efFlatContext, FALSE);
+    efAddConns(&efFlatContext);
+
     efFlatGlob();
 
     return &efFlatContext;
@@ -366,6 +375,9 @@ efFlatNodesDeviceless(hc, cdata)
 	/* Process our own connections and adjustments */
 	efAddConns(hc);
 
+	/* Mark this definition as having no devices, so it will not be visited */
+	hc->hc_use->use_def->def_flags |= DEF_NODEVICES;
+
 	(*usecount)--;
     }
     return (0);
