@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFhier.c,v 1.4 2010/08/25 17:33:56 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFhier.c,v 1.5 2010/12/16 18:59:03 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -107,7 +107,7 @@ efHierSrUses(hc, func, cdata)
 		xbase = xsep * (newhc.hc_x - u->use_xlo);
 		ybase = ysep * (newhc.hc_y - u->use_ylo);
 		GeoTransTranslate(xbase, ybase, &t, &newhc.hc_trans);
-		newhc.hc_hierName = NULL;
+		newhc.hc_hierName = efHNFromUse(&newhc, hc->hc_hierName);
 		if ((*func)(&newhc, cdata))
 		    return (1);
 	    }
