@@ -89,6 +89,7 @@ struct transRec
     int		 tr_gatelen;		/* Perimeter of connection to gate */
     NodeRegion	*tr_gatenode;		/* Node region for gate terminal */
     NodeRegion	*tr_termnode[MAXSD];	/* Node region for each diff terminal */
+    NodeRegion  *tr_subsnode;		/* Substrate node */
     int		 tr_termlen[MAXSD];	/* Length of each diff terminal edge,
 					 * used for computing L/W for the fet.
 					 */
@@ -1000,7 +1001,8 @@ void
 extSeparateBounds(nterm)
     int nterm;			/* last terminal (# terminals - 1) */
 {
-    LinkedBoundary *lb, *lbcomp, *lblast, *lbnext;
+    Rect lbrect;
+    LinkedBoundary *lb, *lbstart, *lbend, *lblast, *lbnext;
     bool found;
 
     /* Avoid crash condition on a badly-defined extract definition */
@@ -1014,7 +1016,8 @@ extSeparateBounds(nterm)
 	extSpecialBounds[nterm]->b_next = NULL;
 
 	/* Add connected segments until no more are found */
-	lbcomp = extSpecialBounds[nterm];
+	lbstart = lbend = extSpecialBounds[nterm];
+	lbrect = lbstart->r;
 	found = TRUE;
 	while (found == TRUE)
 	{
@@ -1026,22 +1029,61 @@ extSeparateBounds(nterm)
 		/* checking the direction of the segment. . . 	*/
 
 		lbnext = lb->b_next;
-		if (((lb->r.r_xbot == lbcomp->r.r_xbot) &&
-		    (lb->r.r_ybot == lbcomp->r.r_ybot)) ||
-		    ((lb->r.r_xtop == lbcomp->r.r_xtop) &&
-		    (lb->r.r_ytop == lbcomp->r.r_ytop)) ||
-		    ((lb->r.r_xtop == lbcomp->r.r_xbot) &&
-		    (lb->r.r_ytop == lbcomp->r.r_ybot)) ||
-		    ((lb->r.r_xbot == lbcomp->r.r_xtop) &&
-		    (lb->r.r_ybot == lbcomp->r.r_ytop)))
+		if (((lb->r.r_xbot == lbrect.r_xbot) &&
+		    (lb->r.r_ybot == lbrect.r_ybot)))
 		{
 		    if (lblast == NULL)
 			extSpecialBounds[0] = lb->b_next;
 		    else
 			lblast->b_next = lb->b_next;
-		    lbcomp->b_next = lb;
-		    lbcomp = lb;
-		    lb->b_next = NULL;
+		    // Insert lb after lbstart
+		    lb->b_next = lbstart->b_next;
+		    lbstart->b_next = lb;
+		    lbstart = lb;
+		    lbrect.r_xbot = lb->r.r_xtop;
+		    lbrect.r_ybot = lb->r.r_ytop;
+		    found = TRUE;
+		}
+		else if (((lb->r.r_xtop == lbrect.r_xbot) &&
+		    (lb->r.r_ytop == lbrect.r_ybot)))
+		{
+		    if (lblast == NULL)
+			extSpecialBounds[0] = lb->b_next;
+		    else
+			lblast->b_next = lb->b_next;
+		    lb->b_next = lbstart->b_next;
+		    lbstart->b_next = lb;
+		    lbstart = lb;
+		    lbrect.r_xbot = lb->r.r_xbot;
+		    lbrect.r_ybot = lb->r.r_ybot;
+		    found = TRUE;
+		}
+		else if (((lb->r.r_xtop == lbrect.r_xtop) &&
+		    (lb->r.r_ytop == lbrect.r_ytop)))
+		{
+		    if (lblast == NULL)
+			extSpecialBounds[0] = lb->b_next;
+		    else
+			lblast->b_next = lb->b_next;
+		    lb->b_next = lbend->b_next;
+		    lbend->b_next = lb;
+		    lbend = lb;
+		    lbrect.r_xtop = lb->r.r_xbot;
+		    lbrect.r_ytop = lb->r.r_ybot;
+		    found = TRUE;
+		}
+		else if (((lb->r.r_xbot == lbrect.r_xtop) &&
+		    (lb->r.r_ybot == lbrect.r_ytop)))
+		{
+		    if (lblast == NULL)
+			extSpecialBounds[0] = lb->b_next;
+		    else
+			lblast->b_next = lb->b_next;
+		    lb->b_next = lbend->b_next;
+		    lbend->b_next = lb;
+		    lbend = lb;
+		    lbrect.r_xtop = lb->r.r_xtop;
+		    lbrect.r_ytop = lb->r.r_ytop;
 		    found = TRUE;
 		}
 		else
@@ -1165,6 +1207,7 @@ extOutputTrans(def, transList, outFile)
 	extTransRec.tr_nterm = 0;
 	extTransRec.tr_gatelen = 0;
 	extTransRec.tr_perim = 0;
+	extTransRec.tr_subsnode = (NodeRegion *)NULL;
 
 	arg.fra_def = def;
 	arg.fra_connectsTo = ExtCurStyle->exts_transConn;
@@ -1235,7 +1278,7 @@ extOutputTrans(def, transList, outFile)
 	 */
 	subsName = ExtCurStyle->exts_transSubstrateName[t];
 	if (!TTMaskIsZero(&ExtCurStyle->exts_transSubstrateTypes[t])
-		&& (subsNode = extTransFindSubsNode(def, reg)))
+		&& (subsNode = extTransRec.tr_subsnode))
 	{
 	    subsName = extNodeName(subsNode);
 	}
@@ -1694,43 +1737,46 @@ extOutputTrans(def, transList, outFile)
     }
 }
 
-NodeRegion *
-extTransFindSubsNode(def, treg)
+int
+extTransFindSubsFunc1(tile, def, sn)
+    Tile *tile;
     CellDef *def;
-    TransRegion *treg;
+    NodeRegion **sn;
 {
-    TileType t = TiGetType(treg->treg_tile);
+    TileType t;
     TileTypeBitMask *mask;
-    NodeRegion *nreg;
     Rect tileArea;
     int pNum;
+    int extTransFindSubsFunc2();	/* Forward declaration */
 
-    TiToRect(treg->treg_tile, &tileArea);
+    TiToRect(tile, &tileArea);
+    if (IsSplit(tile))
+        t = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
+    else
+        t = TiGetTypeExact(tile);
     mask = &ExtCurStyle->exts_transSubstrateTypes[t];
     for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
     {
 	if (TTMaskIntersect(&DBPlaneTypes[pNum], mask))
 	{
 	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum], &tileArea,
-		    mask, extTransFindSubsFunc, (ClientData) &nreg))
-		return nreg;
+		    mask, extTransFindSubsFunc2, (ClientData)sn))
+		return 1;
 	}
     }
-
-    return (NodeRegion *) NULL;
+    return 0;
 }
 
 int
-extTransFindSubsFunc(tile, pnreg)
+extTransFindSubsFunc2(tile, sn)
     Tile *tile;
-    NodeRegion **pnreg;
+    NodeRegion **sn;
 {
     if (tile->ti_client != (ClientData) extUnInit)
     {
-	*pnreg = (NodeRegion *) tile->ti_client;
+	*sn = (NodeRegion *) tile->ti_client;
 	return 1;
     }
-
     return 0;
 }
 
@@ -1753,12 +1799,14 @@ extTransFindSubsFunc(tile, pnreg)
  */
 
 int
-extTransTileFunc(tile, pNum)
+extTransTileFunc(tile, pNum, arg)
     Tile *tile;
     int pNum;
+    FindRegion *arg;
 {
     TileTypeBitMask mask;
     TileType loctype;
+    int perim;
 
     LabelList *ll;
     Label *lab;
@@ -1790,7 +1838,17 @@ extTransTileFunc(tile, pNum)
 
     mask = ExtCurStyle->exts_transConn[loctype];
     TTMaskCom(&mask);
-    (void) extEnumTilePerim(tile, mask, pNum, extTransPerimFunc, (ClientData)NULL);
+
+    /* NOTE:  DO NOT USE extTransRec.tr_perim += extEnumTilePerim(...)	*/
+    /* The AMD target gcc compile works and the Intel target gcc	*/
+    /* compile doesn't!  The following code works the same on both.	*/
+
+    perim = extEnumTilePerim(tile, mask, pNum,
+		extTransPerimFunc, (ClientData)NULL);
+    extTransRec.tr_perim += perim;
+
+    if (extTransRec.tr_subsnode == (NodeRegion *)NULL)
+	extTransFindSubsFunc1(tile, arg->fra_def, &extTransRec.tr_subsnode);
     return (0);
 }
 
@@ -1977,7 +2035,7 @@ extAnnularTileFunc(tile, pNum)
 
     mask = ExtCurStyle->exts_transConn[loctype];
     TTMaskCom(&mask);
-    (void) extEnumTilePerim(tile, mask, pNum, extSpecialPerimFunc, (ClientData) TRUE);
+    extEnumTilePerim(tile, mask, pNum, extSpecialPerimFunc, (ClientData) TRUE);
     return (0);
 }
 
@@ -2481,9 +2539,9 @@ extTransFirst(tile, arg)
     reg = (TransRegion *) mallocMagic((unsigned) (sizeof (TransRegion)));
     reg->treg_next = (TransRegion *) NULL;
     reg->treg_labels = (LabelList *) NULL;
-    reg->treg_pnum = DBNumPlanes;
     reg->treg_area = 0;
     reg->treg_tile = tile;
+    reg->treg_pnum = DBNumPlanes;
 
     if (IsSplit(tile))
 	reg->treg_type = SplitSide(tile) ? SplitRightType(tile) : SplitLeftType(tile);
@@ -2514,12 +2572,16 @@ extTransEach(tile, pNum, arg)
 	reg->treg_type = TiGetTypeExact(tile);
     }
 
-    /* Don't double-count area on contacts (added by Tim 1/9/07) */
-    if (pNum == DBPlane(TiGetType(tile)))
-	reg->treg_area += area;
+    /* The following is non-ideal.  It assumes that the lowest plane of	*/
+    /* types connected to a device is the plane of the device itself.	*/
+    /* Otherwise, the area of the device will be miscalculated.		*/
+    
+    if (pNum < reg->treg_pnum) reg->treg_area = 0;
 
     extSetNodeNum((LabRegion *) reg, pNum, tile);
 
+    if (pNum == reg->treg_pnum) reg->treg_area += area;
+
     return (0);
 }
 
@@ -2746,6 +2808,12 @@ topside:
 		{
 		    PUSHTILEBOTTOM(tp, tilePlaneNum);
 		}
+		else if (tp->ti_client != (ClientData)reg && TTMaskHasType(mask, t))
+		{
+		    /* Count split tile twice, once for each node it belongs to. */
+		    tp->ti_client = extUnInit;
+		    PUSHTILEBOTTOM(tp, tilePlaneNum);
+		}
 	    }
             else
 	    {
@@ -2785,6 +2853,12 @@ leftside:
 		{
 		    PUSHTILERIGHT(tp, tilePlaneNum);
 		}
+		else if (tp->ti_client != (ClientData)reg && TTMaskHasType(mask, t))
+		{
+		    /* Count split tile twice, once for each node it belongs to. */
+		    tp->ti_client = extUnInit;
+		    PUSHTILERIGHT(tp, tilePlaneNum);
+		}
 	    }
             else
 	    {
@@ -2825,6 +2899,12 @@ bottomside:
 		{
 		    PUSHTILETOP(tp, tilePlaneNum);
 		}
+		else if (tp->ti_client != (ClientData)reg && TTMaskHasType(mask, t))
+		{
+		    /* Count split tile twice, once for each node it belongs to. */
+		    tp->ti_client = extUnInit;
+		    PUSHTILETOP(tp, tilePlaneNum);
+		}
 	    }
             else
 	    {
@@ -2864,6 +2944,12 @@ rightside:
 		{
 		    PUSHTILELEFT(tp, tilePlaneNum);
 		}
+		else if (tp->ti_client != (ClientData)reg && TTMaskHasType(mask, t))
+		{
+		    /* Count split tile twice, once for each node it belongs to	*/
+		    tp->ti_client = extUnInit;
+		    PUSHTILELEFT(tp, tilePlaneNum);
+		}
 	    }
             else
 	    {
@@ -2958,6 +3044,7 @@ donesides:
 	if (pMask = DBAllConnPlanes[type])
 	{
 	    Rect biggerArea;
+	    bool is_split = IsSplit(tile);
 
 	    extNbrUn = extUnInit;
 	    TITORECT(tile, &pla.area);
@@ -2966,9 +3053,17 @@ donesides:
 		if ((pNum != tilePlaneNum) && PlaneMaskHasPlane(pMask, pNum))
 		{
 		    pla.plane = pNum;
-		    (void) DBSrPaintArea((Tile *) NULL,
-			    arg->fra_def->cd_planes[pNum], &biggerArea,
-			    mask, extNbrPushFunc, (ClientData) &pla);
+		    if (is_split)
+		        DBSrPaintNMArea((Tile *) NULL,
+				arg->fra_def->cd_planes[pNum],
+				TiGetTypeExact(tile) &
+				(TT_DIAGONAL | TT_SIDE | TT_DIRECTION),
+				&biggerArea, mask, extNbrPushFunc,
+				(ClientData) &pla);
+		    else
+		        DBSrPaintArea((Tile *) NULL,
+				arg->fra_def->cd_planes[pNum], &biggerArea,
+				mask, extNbrPushFunc, (ClientData) &pla);
 		}
 	}
     }
