@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtSubtree.c,v 1.2 2009/05/30 03:14:00 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtSubtree.c,v 1.3 2010/06/24 12:37:17 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -741,7 +741,6 @@ extSubtreeTileToNode(tp, pNum, et, ha, doHard)
 	 * Make sure we've got a valid tile -- interactions with interrupts
 	 * can cause problems.
 	 */
-#ifdef NONMANHATTAN
 	if (IsSplit(tp))
 	{
 	    if (SplitSide(tp))
@@ -750,12 +749,10 @@ extSubtreeTileToNode(tp, pNum, et, ha, doHard)
 		ttype = SplitLeftType(tp);
 	}
 	else
-#endif
-	ttype = TiGetTypeExact(tp);
+	    ttype = TiGetTypeExact(tp);
 
 	if (pNum >= PL_PAINTBASE)
 	{
-#ifdef NONMANHATTAN
 	    if (IsSplit(tp))
 	    {
 		if (DBSrPaintNMArea((Tile *) NULL,
@@ -770,7 +767,6 @@ extSubtreeTileToNode(tp, pNum, et, ha, doHard)
 	    }
 	    else
 	    {
-#endif
 		if (DBSrPaintArea((Tile *) NULL,
 			et->et_lookNames->cd_planes[pNum], &r,
 			&DBAllButSpaceBits, extConnFindFunc, (ClientData) &reg))
@@ -779,9 +775,7 @@ extSubtreeTileToNode(tp, pNum, et, ha, doHard)
 			return ("(none)");
 		    return (extNodeName(reg));
 		}
-#ifdef NONMANHATTAN
 	    }
-#endif
 	}
     }
 
@@ -881,7 +875,6 @@ extSubtreeHardNode(tp, pNum, et, ha)
 
     ASSERT(lreg->lreg_labels == NULL, "extSubtreeHardNode");
 
-#ifdef NONMANHATTAN
     if (IsSplit(tp))
     {
 	if (SplitSide(tp))
@@ -890,7 +883,6 @@ extSubtreeHardNode(tp, pNum, et, ha)
 	    ttype = SplitLeftType(tp);
     }
     else
-#endif
 	ttype = TiGetTypeExact(tp);
 
     arg.hw_ha = ha;
