@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtUnique.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtUnique.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -247,10 +247,11 @@ makeUnique:
 
 	    lab = ll2->ll_label;
 	    saveLab = *lab;
-	    DBEraseLabelsByContent(def, &lab->lab_rect,
-		    lab->lab_pos, lab->lab_type, lab->lab_text);
-	    (void) DBPutLabel(def, &saveLab.lab_rect,
-		    saveLab.lab_pos, name2, saveLab.lab_type, saveLab.lab_flags);
+	    DBEraseLabelsByContent(def, &lab->lab_rect, lab->lab_type, lab->lab_text);
+	    (void) DBPutFontLabel(def, &saveLab.lab_rect,
+		 	saveLab.lab_font, saveLab.lab_size, saveLab.lab_rotate,
+			&saveLab.lab_offset, saveLab.lab_just, name2,
+			saveLab.lab_type, saveLab.lab_flags);
 	    ll2->ll_label = (Label *) NULL;
 	}
 
