@@ -4,7 +4,7 @@
  * Defines things shared internally by the extract module of Magic,
  * but not generally needed outside the extract module.
  *
- * rcsid "$Header: /usr/cvsroot/magic-8.0/extract/extractInt.h,v 1.5 2009/05/30 03:14:00 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-8.0/extract/extractInt.h,v 1.7 2010/08/10 00:18:46 tim Exp $"
  *
  *     ********************************************************************* 
  *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
@@ -274,10 +274,10 @@ typedef struct
 
 /* Directions in which we can be following the boundary of a perimeter	*/
 
-#define	BD_LEFT		0	/* Inside is to right */
-#define	BD_TOP		1	/* Inside is below */
-#define	BD_RIGHT	2	/* Inside is to left */
-#define	BD_BOTTOM	3	/* Inside is above */
+#define	BD_LEFT		1	/* Inside is to right */
+#define	BD_TOP		2	/* Inside is below */
+#define	BD_RIGHT	4	/* Inside is to left */
+#define	BD_BOTTOM	8	/* Inside is above */
 
 /* -------- Yank buffers for hierarchical and array extraction -------- */
 
@@ -896,8 +896,6 @@ extern ClientData extUnInit;
 /* For non-recursive flooding algorithm */
 #define	VISITPENDING	((ClientData) NULL)	/* Marks tiles on stack */
 
-#ifdef NONMANHATTAN
-
 /* Note that this macro depends on MAXPLANES being small	*/
 /* compared to the bit position of TT_SIDE.  Since tens of	*/
 /* thousands of planes is inconceivable, this should not be a	*/
@@ -945,19 +943,6 @@ extern ClientData extUnInit;
 	STACKPUSH((ClientData)(pointertype)(pl | TT_SIDE), extNodeStack); \
 	STACKPUSH((ClientData)(pointertype)tp, extNodeStack)
 
-#else
-
-#define	PUSHTILE(tp, pl) \
-	(tp)->ti_client = VISITPENDING; \
-	STACKPUSH((ClientData)pl, extNodeStack); \
-	STACKPUSH((ClientData)tp, extNodeStack)
-
-#define POPTILE(tp, pl) \
-	tp = (Tile *) STACKPOP(extNodeStack); \
-	pl = (int) STACKPOP(extNodeStack)
-
-#endif
-
 /* ------------------------- Region finding --------------------------- */
 
 extern Region *ExtFindRegions();
