@@ -489,7 +489,7 @@ TOGLEventProc(clientData, xevent)
 		}
 		nbytes = 0;
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		if (mw && (mw->w_flags & WIND_SCROLLBARS))
@@ -512,7 +512,7 @@ TOGLEventProc(clientData, xevent)
 
 		if (IsModifierKey(keysym)) break;	/* Don't handle modifiers */
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 keys_and_buttons:
@@ -776,7 +776,7 @@ keys_and_buttons:
 		width = ConfigureEvent->width;
 		height = ConfigureEvent->height;
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		screenRect.r_xbot = ConfigureEvent->x;
@@ -795,7 +795,7 @@ keys_and_buttons:
 	    {
 		XVisibilityEvent *VisEvent = (XVisibilityEvent*) xevent;
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		switch(VisEvent->state)
@@ -824,7 +824,7 @@ keys_and_buttons:
 		XExposeEvent *ExposeEvent = (XExposeEvent*) xevent;
 		Rect screenRect;
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		screenRect.r_xbot = ExposeEvent->x;
@@ -944,9 +944,7 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     grSetStipplePtr = grtoglSetStipple;
     grSetLineStylePtr = grtoglSetLineStyle;
     grSetCharSizePtr = grtoglSetCharSize;
-#ifdef NONMANHATTAN
     grFillPolygonPtr = grtoglFillPolygon;
-#endif
 
     GrFreeBackingStorePtr = grtkFreeBackingStore;
     GrCreateBackingStorePtr = grtkCreateBackingStore;
@@ -1074,7 +1072,7 @@ GrTOGLCreate(w, name)
 
 	w->w_grdata = (ClientData) tkwind;
 	
-	entry = HashFind(&grTOGLWindowTable, tkwind);
+	entry = HashFind(&grTOGLWindowTable, (char *)tkwind);
 	HashSetValue(entry,w);
 
 	/* ensure that the visual is what we wanted, if possible to change */
@@ -1147,7 +1145,7 @@ GrTOGLDelete(w)
     HashEntry	*entry;
 
     xw = (Tk_Window) w->w_grdata;
-    entry = HashLookOnly(&grTOGLWindowTable, xw);
+    entry = HashLookOnly(&grTOGLWindowTable, (char *)xw);
     HashSetValue(entry,NULL);
 
     Tcl_DeleteCommand(magicinterp, Tk_PathName(xw));
@@ -1422,7 +1420,7 @@ GrTOGLWindowId(tkname)
     tkwind = Tk_NameToWindow(magicinterp, tkname, Tk_MainWindow(magicinterp));
     if (tkwind != NULL)
     {   
-	entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+	entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 	mw = (entry) ? (MagWindow *)HashGetValue(entry) : 0;
 	if (mw) id = mw->w_wid;
     }       
