@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/defWrite.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/defWrite.c,v 1.2 2008/02/10 19:30:21 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -608,7 +608,7 @@ defnodeVisit(node, res, cap, defdata)
     FILE *f = defdata->f;
     CellDef *def = defdata->def;
     float oscale = defdata->scale;
-    TileTypeBitMask tmask;
+    TileTypeBitMask tmask, *rmask;
     TileType magictype;
     EFNodeName *thisnn;
     int defNetGeometryFunc();		/* Forward declaration */
@@ -716,7 +716,8 @@ defnodeVisit(node, res, cap, defdata)
 	/* Add the residue types to any contact type */
 	if (DBIsContact(magictype))
 	{
-	    TTMaskSetMask(&tmask, DBResidueMask(magictype));
+	    rmask = DBResidueMask(magictype);
+	    TTMaskSetMask(&tmask, rmask);
 	    TTMaskSetType(&tmask, magictype);
 	}
 	/* Expand the rectangle around the port to overlap any	*/
@@ -781,7 +782,7 @@ defNetGeometryFunc(tile, plane, defdata)
     FILE *f = defdata->f;
     CellDef *def = defdata->def;
     float oscale = defdata->scale;
-    TileTypeBitMask *rMask;
+    TileTypeBitMask *rMask, *r2Mask;
     TileType rtype, r2type, ttype = TiGetType(tile);
     Rect r;
     unsigned char orient;
@@ -837,8 +838,11 @@ defNetGeometryFunc(tile, plane, defdata)
 		GeoInclude(&r2, &r);
 	    }
 	    else if (r2type >= DBNumUserLayers)
-		if (TTMaskHasType(DBResidueMask(r2type), ttype))
+	    {
+		r2Mask = DBResidueMask(r2type);
+		if (TTMaskHasType(r2Mask, ttype))
 		    return 0;
+	    }
 	}
 
 	for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp)) /* Left */
@@ -851,8 +855,11 @@ defNetGeometryFunc(tile, plane, defdata)
 		GeoInclude(&r2, &r);
 	    }
 	    else if (r2type >= DBNumUserLayers)
-		if (TTMaskHasType(DBResidueMask(r2type), ttype))
+	    {
+		r2Mask = DBResidueMask(r2type);
+		if (TTMaskHasType(r2Mask, ttype))
 		    return 0;
+	    }
 	}
 
 	for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp = TR(tp)) /* Bottom */
@@ -865,8 +872,11 @@ defNetGeometryFunc(tile, plane, defdata)
 		GeoInclude(&r2, &r);
 	    }
 	    else if (r2type >= DBNumUserLayers)
-		if (TTMaskHasType(DBResidueMask(r2type), ttype))
+	    {
+		r2Mask = DBResidueMask(r2type);
+		if (TTMaskHasType(r2Mask, ttype))
 		    return 0;
+	    }
 	}
 
 	for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp)) /* Right */
@@ -879,8 +889,11 @@ defNetGeometryFunc(tile, plane, defdata)
 		GeoInclude(&r2, &r);
 	    }
 	    else if (r2type >= DBNumUserLayers)
-		if (TTMaskHasType(DBResidueMask(r2type), ttype))
+	    {
+		r2Mask = DBResidueMask(r2type);
+		if (TTMaskHasType(r2Mask, ttype))
 		    return 0;
+	    }
 	}
     }
 
@@ -1344,7 +1357,7 @@ defCountViaFunc(tile, cviadata)
     CViaData *cviadata;
 {
     TileType ttype = TiGetType(tile), ctype, rtype;
-    TileTypeBitMask *rmask;
+    TileTypeBitMask *rmask, *rmask2;
     Tile *tp;
     char *lname, vname[100], *vp;
     Rect r, r2;
@@ -1396,8 +1409,11 @@ defCountViaFunc(tile, cviadata)
 	    GeoInclude(&r2, &r);
 	}
 	else if (rtype >= DBNumUserLayers)
-	    if (TTMaskHasType(DBResidueMask(rtype), ctype))
+	{
+	    rmask2 = DBResidueMask(rtype);
+	    if (TTMaskHasType(rmask2, ctype))
 		return 0;
+	}
     }
 
     /* Left */
@@ -1411,8 +1427,11 @@ defCountViaFunc(tile, cviadata)
 	    GeoInclude(&r2, &r);
 	}
 	else if (rtype >= DBNumUserLayers)
-	    if (TTMaskHasType(DBResidueMask(rtype), ctype))
+	{
+	    rmask2 = DBResidueMask(rtype);
+	    if (TTMaskHasType(rmask2, ctype))
 		return 0;
+	}
     }
 
     /* Bottom */
@@ -1426,8 +1445,11 @@ defCountViaFunc(tile, cviadata)
 	    GeoInclude(&r2, &r);
 	}
 	else if (rtype >= DBNumUserLayers)
-	    if (TTMaskHasType(DBResidueMask(rtype), ctype))
+	{
+	    rmask2 = DBResidueMask(rtype);
+	    if (TTMaskHasType(rmask2, ctype))
 		return 0;
+	}
     }
 
     /* Right */
@@ -1441,8 +1463,11 @@ defCountViaFunc(tile, cviadata)
 	    GeoInclude(&r2, &r);
 	}
 	else if (rtype >= DBNumUserLayers)
-	    if (TTMaskHasType(DBResidueMask(rtype), ctype))
+	{
+	    rmask2 = DBResidueMask(rtype);
+	    if (TTMaskHasType(rmask2, ctype))
 		return 0;
+	}
     }
 
     /* All values for the via rect are in 1/2 lambda to account */
