@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/netmenu/NMnetlist.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/netmenu/NMnetlist.c,v 1.2 2010/09/12 23:36:13 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -444,8 +444,8 @@ NMNewNetlist(name)
     }
     (void) StrDup(&new->nl_fileName, fullName);
     if ((fgets(line, MAXLINESIZE, file) == NULL)
-	|| ((strcmp(line, " Net List File\n") != 0) /* Backward compatibility*/
-	&& (strcmp(line, " Netlist File\n") != 0)))
+	|| ((strcasecmp(line, " Net List File\n") != 0) /* Backward compatibility*/
+	&& (strcasecmp(line, " Netlist File\n") != 0)))
     {
 	TxError("%s isn't a legal netlist file.\n", new->nl_fileName);
 	TxError("Creating new netlist.\n");
