@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plow/PlowRules2.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plow/PlowRules2.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1094,7 +1094,7 @@ plowFoundCell(cellTile, ar)
 	}
 
 	/* Only queue the edge if the cell has not moved far enough */
-	if ((use->cu_client != (ClientData)MINFINITY) &&
+	if ((use->cu_client != (ClientData)CLIENTDEFAULT) &&
 		((int)(use->cu_client) < xmove))
 	{
 	    edge.e_use = use;
