@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResRex.c,v 1.2 2009/08/03 13:41:27 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResRex.c,v 1.3 2010/03/08 13:33:33 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -12,7 +12,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 /* It is hard to get the value of MAXFLOAT in a portable manner. */
 #if defined(ibm032)
 #define MAXFLOAT        ((float)3.40282346638528860e+38)
-#elif !defined(__NetBSD__) && !defined(__FreeBSD__) && !defined(CYGWIN) && !defined(macosx) 
+#elif !defined(__NetBSD__) && !defined(__FreeBSD__) && !defined(CYGWIN) && !defined(macosx) && !defined(__DragonFly__)
 #include <values.h>
 #endif
 
