@@ -238,28 +238,36 @@ SimFreeNodeRegs()
  */
 int SimInitConnTables()
 {
-    int  t, sd, p;
+    int  i, t, sd, p;
 
     SimTransMask = ExtCurStyle->exts_transMask;
 
     TTMaskZero( &SimSDMask );
     for( t = TT_TECHDEPBASE; t < DBNumTypes; t++ )
     {
-	TTMaskSetMask( &SimSDMask, &ExtCurStyle->exts_transSDTypes[t] );
-	TTMaskZero( &SimFetMask[t] );
+	for (i = 0; !TTMaskHasType(&ExtCurStyle->exts_transSDTypes[t][i],
+			TT_SPACE); i++)
+	{
+	     TTMaskSetMask( &SimSDMask, &ExtCurStyle->exts_transSDTypes[t][i] );
+	     TTMaskZero( &SimFetMask[t] );
+	}
     }
 
     SimFetPlanes = 0;
-    for( t = TT_TECHDEPBASE; t < DBNumTypes; t++ )
+    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
     {
-	if( TTMaskHasType( &SimTransMask, t ) )
+	if (TTMaskHasType(&SimTransMask, t))
 	{
-	    for( sd = TT_TECHDEPBASE; sd < DBNumTypes; sd++ )
+	    for (sd = TT_TECHDEPBASE; sd < DBNumTypes; sd++)
 	    {
-		if( TTMaskHasType( &ExtCurStyle->exts_transSDTypes[t], sd ) )
+		for (i = 0; !TTMaskHasType(&ExtCurStyle->exts_transSDTypes[t][i],
+				TT_SPACE); i++)
 		{
-		    TTMaskSetType( &SimFetMask[sd], t );
-		    SimFetPlanes |= PlaneNumToMaskBit( DBPlane(t) );
+		    if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[t][i], sd))
+		    {
+			TTMaskSetType(&SimFetMask[sd], t);
+			SimFetPlanes |= PlaneNumToMaskBit(DBPlane(t));
+		    }
 		}
 	    }
 	}
@@ -361,7 +369,6 @@ int SimTransTerms( bp, trans )
     int		pNum;
     int		i;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile)) {
 	switch(bp->b_direction) {
 	    case BD_LEFT:
@@ -379,7 +386,6 @@ int SimTransTerms( bp, trans )
 	}
     }
     else
-#endif
 	type = TiGetTypeExact(tile);
 
     pNum = DBPlane(type);
@@ -456,18 +462,26 @@ int SimTermNum( trans, reg )
 }
 
 
-int SimTransistorTile( tile, pNum, arg )
+int
+SimTransistorTile(tile, pNum, arg)
     Tile	*tile;
     int		pNum;
     FindRegion	*arg;
 {
-    extSetNodeNum( (LabRegion *) &transistor, pNum, tile );
-    if( transistor.t_do_terms )
-	(void) extEnumTilePerim( tile,
-	  ExtCurStyle->exts_transSDTypes[TiGetType( tile )],
-	  SimTransTerms, (ClientData) &transistor );
+    int i;
+    TileType t;
 
-    return( 0 );
+    extSetNodeNum((LabRegion *)&transistor, pNum, tile);
+    if (transistor.t_do_terms)
+    {
+	t = TiGetType(tile);
+	for (i = 0; !TTMaskHasType(&ExtCurStyle->exts_transSDTypes[t][i],
+			TT_SPACE); i++)
+	    extEnumTilePerim(tile, ExtCurStyle->exts_transSDTypes[t][i],
+			SimTransTerms, (ClientData) &transistor );
+    }
+
+    return (0);
 }
 
 
@@ -550,11 +564,9 @@ SimFindOneNode( sx, tile )
     arg.fra_connectsTo = ExtCurStyle->exts_nodeConn;
     arg.fra_def = def;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
 	type = SplitSide(tile) ? TiGetRightType(tile) : TiGetLeftType(tile);
     else
-#endif
 	type = TiGetTypeExact(tile);
 
     arg.fra_pNum = DBPlane(type);
@@ -572,12 +584,10 @@ SimFindOneNode( sx, tile )
 	gateTile->ti_client = (ClientData) extUnInit;
 	arg.fra_connectsTo = &SimTransMask;
 
-#ifdef NONMANHATTAN
 	if (IsSplit(tile))
 	    loctype = SplitSide(gateTile) ? TiGetRightType(gateTile)
 			: TiGetLeftType(gateTile);
 	else
-#endif
 	    loctype = TiGetTypeExact(gateTile);
 
 	arg.fra_pNum = DBPlane(loctype);
@@ -613,12 +623,10 @@ SimFindOneNode( sx, tile )
 	/* collect the transistor position, and its terminals */
 	arg.fra_connectsTo = &SimTransMask;
 
-#ifdef NONMANHATTAN
 	if (IsSplit(tile))
 	    loctype = SplitSide(sdTile) ? TiGetRightType(sdTile)
 			: TiGetLeftType(sdTile);
 	else
-#endif
 	    loctype = TiGetTypeExact(sdTile);
 
 	arg.fra_pNum = DBPlane(loctype);
