@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/signals.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/signals.c,v 1.2 2010/03/08 13:33:34 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -669,7 +669,7 @@ SigInit()
 void
 sigSetAction(int signo, sigRetVal (*handler)(int))
 {
-#if defined(SYSV) || defined(CYGWIN)
+#if defined(SYSV) || defined(CYGWIN) || defined(__NetBSD__)
   struct sigaction sa;
 
   sa.sa_handler = handler;
