@@ -16,7 +16,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/windows/windDisp.c,v 1.3 2001/01/12 22:13:35 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/windows/windDisp.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -329,6 +329,15 @@ WindAreaChanged(w, area)
 	    (ClientData) w->w_clipAgainst->r_next);
     }
     UndoEnable();
+
+    /* If the area is NULL or encompasses the whole screen area, and	*/
+    /* there is no backing store, then we should create it so that it	*/
+    /* will be copied into on the next display redraw.			*/
+
+    if ((w != NULL) && (w->w_backingStore == (ClientData)NULL) &&
+		(!(w->w_flags & WIND_OBSCURED)) && (GrCreateBackingStorePtr != NULL))
+	if ((area == (Rect *)NULL) || GEO_SURROUND(&biggerArea, &w->w_screenArea))
+	    (*GrCreateBackingStorePtr)(w);
 }
 
 int
@@ -435,7 +444,7 @@ windBarLocations(w, leftBar, botBar, up, down, right, left, zoom)
 
 /*
  * ----------------------------------------------------------------------------
- * windDrawBorder --
+ * WindDrawBorder --
  *
  *	Draw the border of windows.  A window lock is created & then destroyed.
  *
@@ -448,7 +457,7 @@ windBarLocations(w, leftBar, botBar, up, down, right, left, zoom)
  */
 
 void
-windDrawBorder(w, clip)
+WindDrawBorder(w, clip)
     MagWindow *w;
     Rect *clip;
 {
@@ -656,9 +665,13 @@ windNewView(w)
     MagWindow *w;
 {
     Rect leftBar, botBar, up, down, right, left, zoom;
-    windBarLocations(w, &leftBar, &botBar, &up, &down, &right, &left, &zoom);
-    WindAreaChanged(w, &leftBar);
-    WindAreaChanged(w, &botBar);
+
+    if ((w->w_flags & WIND_SCROLLBARS) != 0)
+    {
+	windBarLocations(w, &leftBar, &botBar, &up, &down, &right, &left, &zoom);
+	WindAreaChanged(w, &leftBar);
+	WindAreaChanged(w, &botBar);
+    }
 }
 
 
@@ -906,7 +919,7 @@ windUpdateFunc(tile, w)
 	&& (w->w_screenArea.r_ybot <= area.r_ybot)
 	&& (w->w_screenArea.r_ytop+1 >= area.r_ytop))) {
 	/* Redisplay the border areas. */
-	windDrawBorder(w, &area);
+	WindDrawBorder(w, &area);
     };
 
     /* Now call the client to redisplay the interior of the window. */
